#!/usr/bin/python
import httplib
import psycopg2
import socket
from datetime import datetime
import hashlib
import magic

SENSOR_NAME = 'httpmole2'
SOCKET_TIMEOUT = 10	

def connect(cur, bot, cmd):
	query = 'SELECT address, port FROM cncs, http_bots WHERE http_bots.id = %s AND cncs.id = http_bots.cnc'
	cur.execute(query, [ bot ])
	row = cur.fetchone()
	address = row[0]
	port = int(row[1])
	query = 'SELECT url, method, version, headers FROM http_cmds WHERE id = %s'
	cur.execute(query, [ cmd ])
	url, method, version, headers = cur.fetchone()
	url = url[url.find('/'):]
	headers_m = {}
	for line in headers.split('\n'):
		k,v = line.split(':', 1)
		headers_m[k] = v
	try:
		conn = httplib.HTTPConnection(address, port)
		conn.request(method, url, None, headers_m)
		response = conn.getresponse()
		rheaders = '\n'.join("%s: %s" %(k,v) for k,v in response.getheaders())
		data = response.read()
		conn.close()
		return response.status, response.reason, rheaders, data
	except socket.error, msg:
		try:
			return msg[0], msg[1], '', ''
		except:
			return 0, msg, '', ''

def add_sample(cur, timestamp, sensor, md5, sample):
	query = 'SELECT md5 FROM samples WHERE md5 = %s'
	cur.execute(query, [ md5 ])
	if cur.rowcount == 0:
		query = 'INSERT INTO samples (md5, sample, size) VALUES (%s, %s, %s)'
		cur.execute(query, [ md5, psycopg2.Binary(sample), len(sample) ])
	query = 'INSERT INTO sightings (md5,experiment,sensor) VALUES (%s, %s, %s)'
	cur.execute(query, [ md5, timestamp, sensor ])
	
def process_bot(db, bot_id):
	cur = db.cursor()
	if bot_id == 65 or bot_id == 282 or bot_id == 284 or bot_id == 404 or bot_id == 405 or bot_id == 6904:
		return False
	print "Connecting to bot %i" %(bot_id)
	query = 'SELECT http_cmd FROM http_bots_ref where http_bot = %s'
	cur.execute(query, [ bot_id ])
	row = cur.fetchone()
	while row != None:
		process_cmd(db, bot_id, row[0])
		row = cur.fetchone()
	db.commit()
	cur.close()

def process_cmd(db, bot_id, cmd_id):
	cur = db.cursor()
	timestamp = datetime.now().strftime('%Y%m%d %H%M%S')
	status, reason, headers, data = connect(cur, bot_id, cmd_id)
	print status, reason
	hasher = hashlib.md5()
	hasher.update(data)
	md5 = hasher.hexdigest()
	newval = False
	query = 'SELECT md5(response) FROM http_replies WHERE http_bot = %s and http_cmd = %s'
	cur.execute(query, [ bot_id, cmd_id ])
	row = cur.fetchone()
	while row != None:
		if row[0] != md5:
			newval = True
			break
		row = cur.fetchone()
	if newval or (cur.rowcount == 0):
		print "Registering new response"
		cur.execute("INSERT INTO http_replies (timestamp, http_bot, http_cmd, sensor, status, reason, headers, response) VALUES ('%s', %i, %i, '%s', %i, '%s', '%s', %s)" %(timestamp, bot_id, cmd_id, SENSOR_NAME, status, reason, headers, psycopg2.Binary(data)))
		rtype = magic.buffer(data)
		if rtype.find('executable PE') != -1:
			print "Adding new sample"
			add_sample(cur, timestamp, SENSOR_NAME, md5, data)
	
if __name__ == '__main__':
	socket.setdefaulttimeout(SOCKET_TIMEOUT)
	magic = magic.open(magic.MAGIC_NONE)
	magic.load()
	db = psycopg2.connect (host='localhost', port='5432', database='malware', user='malware_user', password='malware_password')
	cur = db.cursor()
	cur.execute('SELECT id FROM http_bots')
	count = 0
	row = cur.fetchone()
	while row != None:
		if process_bot(db, row[0]):
			db.commit()
			count = count + 1
		row = cur.fetchone()
	magic.close()
	db.commit()
	cur.close()
	db.close()

