#!/usr/bin/python
from imaplib import IMAP4_SSL
from email.Parser import Parser
import re
import pg
import sys

file_r = re.compile('^Submitted file: ([0-9a-z]+)\r\n',re.M)
analysis_r = re.compile('^Analysis ID: ([0-9]+)\r\n',re.M)
password_r = re.compile('.*password=3D([a-z]+).\r\n',re.M)
result_r = re.compile('^Analysis result: (.*)\r\n',re.M)

subj_r = re.compile("^Analysis result for '([0-9a-z]+)'")
error_r = re.compile('^Unfortunately there was an error while trying to analyze your file.\r\n', re.M)
taskid_r = re.compile('^You can find your report at http://analysis\.seclab\.tuwien\.ac\.at/result\.php\?taskid=([0-9a-z]+)\r\n',re.M)

#Submitted file: c439fa288e0395fa3b95e38d3651f590
#Analysis ID: 216393
#Sample ID: 98245
#Submission ID: 88266
#Your comment:=20
#Analysis result: Processed

def parse_report(report):
	m = file_r.search(report)
	if m == None:
		submitted_file = ''
	else:
		submitted_file = m.group(1)

	m = analysis_r.search(report)
	if m == None:
		analysis_id = ''
	else:
		analysis_id = m.group(1)
	
	m = result_r.search(report)
	if m == None:
		analysis_result = ''
	else:
		analysis_result = m.group(1)
	
	m = password_r.search(report)
	if m == None:
		password = ''
	else:
		password = m.group(1)
	
	return submitted_file, analysis_id, analysis_result, password

def is_duplicate(db, md5):
	q = "SELECT md5 FROM reports WHERE md5 = '%s' and provider::text = 'CWSandbox'" %(md5)
	if db.query(q).ntuples() > 0:
		return True
	else:
		return False

def insert_report(db, md5, id, password):
	q = "INSERT INTO reports (md5, provider, analysis_id, password, available) VALUES ( '%s', 'CWSandbox', %s, '%s', 'true')" %(md5, id, password)
	db.query(q)
	return

def set_invalid(db, md5):
	q = "UPDATE samples SET type = 'invalid' WHERE md5 = '%s'" %(md5)
	db.query(q)
	return

def parse_anubis(subject, report):
	m = error_r.search(report)
	if m != None:
		taskid = ''
	m = taskid_r.search(report)
	if m == None:
		taskid = ''
	else:
		taskid = m.group(1)
	m = subj_r.search(subject)
	if m == None:
		md5 = ''
	else:
		md5 = m.group(1)
	return md5, taskid

def is_anubis(db, md5):
	q = "SELECT md5 FROM reports WHERE md5 = '%s' and provider = 'Anubis'" %(md5)
	if db.query(q).ntuples() > 0:
		return True
	else:
		return False

def update_anubis(db, md5):
	q = "UPDATE reports SET available = 'true' WHERE provider = 'Anubis' and md5 = '%s'" %(md5)
	db.query(q)
	return

def insert_anubis(db,md5,task_id):
	q = "INSERT INTO reports (md5,provider,task_id,available) VALUES ('%s','Anubis','%s',true)" %(md5, task_id)
	db.query(q)

parser = Parser()
conn = pg.connect(host='localhost', port=5432, dbname='malware', user='malware', passwd='malware')
server = IMAP4_SSL("imap.gmail.com", 993)
server.login('malwaredepot@gmail.com','HN@12ls1')
server.select('INBOX')
typ, data = server.search(None, 'ALL')
done_msg = []
invalid_msg = []
for num in data[0].split():
	print "Processing message %s" %(num)
	typ, data = server.fetch(num, '(RFC822)')
	if data == None:
		print "cannot fetch message, skipping..."
		continue
	if data[0] == None:
		print "cannot fetch message, skipping..."
		continue
	mesg = parser.parsestr(data[0][1])
	if mesg['sender'] == 'sandbox@informatik.uni-mannheim.de':
		body = mesg.get_payload()
		md5, id, result, password = parse_report(body)
		if(is_duplicate(conn, md5)):
			print "Duplicate report %s, skipping..." %(md5)
			done_msg.append(num)
			continue
		if(result != 'Processed'):
			print "Invalid report for %s: '%s', skipping..." %(md5, result)
			invalid_msg.append(num)
			set_invalid(conn, md5)
			continue
		insert_report(conn, md5, id, password)
		done_msg.append(num)
	elif mesg['From'] == 'analysis-noreply@seclab.tuwien.ac.at':
		subject = mesg['Subject']
		body = mesg.get_payload()
		md5, taskid = parse_anubis(subject, body)
		if taskid == '' or md5 == '':
			print "Invalid report, skipping..."
			invalid_msg.append(num)
			continue
		if is_anubis(conn, md5):
			update_anubis(conn, md5)
		else:
			insert_anubis(conn, md5, taskid)
		done_msg.append(num)
	else:
		print "sender unknown, skipping..."
sys.stdout.write('Marking processed messages')
for num in done_msg:
	server.copy(num, 'done')
	sys.stdout.write('.')
	sys.stdout.flush()
print
sys.stdout.write('Marking invalid messages')
for num in invalid_msg:
	server.copy(num, 'invalid')
	sys.stdout.write('.')
	sys.stdout.flush()
print
sys.stdout.write('Cleaning up INBOX')
for num in done_msg:
	server.store(num, '+FLAGS.SILENT', '\\Deleted')
	sys.stdout.write('.')
	sys.stdout.flush()
for num in invalid_msg:
	server.store(num, '+FLAGS.SILENT', '\\Deleted')
	sys.stdout.write('.')
	sys.stdout.flush()
print
server.expunge()
server.close()
server.logout()
conn.close()

