#!/usr/bin/python

__author__ = "Jan Goebel <goebel@rz.rwth-aachen.de>"
__date__ = "2007-06-11"
__version__ = "0.1"

import logging
from threading import Thread
import socket
import sys
import re
import Queue
import time
import os
import urllib
import md5
import psycopg2
from datetime import datetime

class connect_job(Thread):
	def __init__(self, dbconn, bot, tname, server, port, nick, usermode='none', channel='none', channel_id = None, password='none'):
		Thread.__init__(self)
		self.dbconn = dbconn
		self.bot = bot
		self.channel_id = channel_id
		self.tname = tname
		self.talkback = False
		self.reconnect = True
		self.init_reconnect = False
		self.sensor_name = 'infiltrator3'
		### logging init
		self.Logger = None
		try_tname = "%s-%i" % (tname, 0)
		logfile = "logs/%s.log" % (try_tname)
		i = 1
		while os.path.exists(logfile):
			try_tname = "%s-%i" % (tname, i)
			logfile = "logs/%s.log" % (try_tname)
			i += 1
		tname = try_tname
		self.Logger = logging.getLogger(tname)
		hdlr = logging.FileHandler(logfile)
		formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
		hdlr.setFormatter(formatter)
		self.Logger.addHandler(hdlr)
		self.Logger.setLevel(10)
		### logging done
		self.running = True
		self.topic = []
		self.priv_messages = {}
		self.url_expre = re.compile('.*((http|https|ftp):\/\/[~@a-zA-Z0-9\-\/\\\.\+:]+).*')
		self.last_list = Queue.Queue(50)
		### init configuration
		self.server = server
		self.port = int(port)
		self.nick = nick
		self.usermode = usermode
		self.channel = channel
		self.password = password
		self.commandList = []
		mess = "thread %s started" % (self.tname)
		self.Logger.info(mess)

	def stop(self):
		self.running = False
		mess = "thread %s stop request" % (self.getName())
		self.Logger.info(mess)

	def run(self):
		### Usermode Set?
		if self.usermode == 'none' or len(self.usermode)<2:
			self.usermode = 'worm worm worm worm:%s' % (self.nick)
			mess = "Setting USER command: %s" % (self.usermode)
			self.Logger.info(mess)

		### Connect to IRC Server
		mess = "Connecting to: %s Port: %i" % (self.server, self.port)
		self.Logger.info(mess)
		self.last_list.put(mess)
		try:
			sobj = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			sobj.connect( (self.server, self.port) )
			sobj.settimeout(5.0)
		except socket.error, e:
			mess = "socket error: %s" % (e)
			self.Logger.info(mess)
			self.last_list.put(mess)
			self.stop()
			return

		### Server Password?
		if self.password != 'none':
			mess = "Sending Server Password: %s" % (self.password)
			self.Logger.info(mess)
			command = "PASS %s\r\n" % (self.password)
			try:
				sobj.send(command)
			except socket.error, e:
				mess = "socket error: %s" % (e)
				self.Logger.info(mess)
				self.stop()
				return
		
		### Sending Nickname
		mess = "Sending Nickname: %s" % (self.nick)
		self.Logger.info(mess)
		command = "NICK %s\r\n" % (self.nick)
		try:
			sobj.send(command)
		except socket.error, e:
			mess = "socket error: %s" % (e)
			self.Logger.info(mess)
			self.stop()
			return

		### Sending Usermode
		mess = "Sending Usermode: %s" % (self.usermode)
		self.Logger.info(mess)
		command = "USER %s\r\n" % (self.usermode)
		try:
			sobj.send(command)
		except socket.error, e:
			mess = "socket error: %s" % (e)
			self.Logger.info(mess)
			self.stop()
			return

		### Push Join to commands
		if self.channel != 'none':
			self.commandList.append("JOIN %s" % (self.channel))
		
		### loop while connected
		connected = True
		self.start_running(self.Logger, sobj, connected)
		if connected:
			sobj.close()
			connected = False
		mess = "thread %s closed" % (self.getName())
		self.Logger.info(mess)
		if self.reconnect and self.init_reconnect:
			mess = "reconnection initiated"
			self.Logger.info(mess)
			self.init_reconnect = False
			self.run()
		try:
			self.Logger.shutdown()
		except AttributeError:
			pass
		return

	def toggle_talkback(self):
		self.talkback = not self.talkback
		return self.talkback

	def get_topics(self):
		return self.topic

	def get_privmsg(self):
		return self.priv_messages

	def get_last(self):
		return self.last_list.queue

	def read(self):
		running = True
		try:
			last_entry = ""
			try:
				last_entry = self.last_list.queue[-1]
				print "\t%s" % (last_entry)
			except:
				pass
			while running:
				if last_entry != self.last_list.queue[-1]:
					last_entry = self.last_list.queue[-1]
					print "\t%s" % (last_entry)
				else:
					time.sleep(1)
		except KeyboardInterrupt:
			running = False
		return

	def join_chan(self, channel):
		self.commandList.append("JOIN %s" % (channel))
		return

	def ping_host(self):
		self.commandList.append("PING :123456")
		return

	def start_running(self, Logger, sobj, connected):
		readbuffer = ""
		LogLine = True
		cur = self.dbconn.cursor()
		while self.running:
			time.sleep(.0001)
			try:
				readbuffer = readbuffer + sobj.recv(1024)
			except socket.error, e:
				if str(e) == 'timed out':
					while len(self.commandList)>0:
						command = self.commandList.pop(0)
						mess = "Sending Command: %s" % (command)
						self.Logger.info(mess)
						sobj.send("%s\r\n" % (command))
					if not self.running:
						break
				else:
					mess = "socket error: %s" % (e)
					self.Logger.info(mess)
					if not self.last_list.full():
						self.last_list.put(mess)
					else:
						self.last_list.get()
						self.last_list.put(mess)
					self.Logger.info("setting cnc %s as inactive" %(self.tname[3:]))
					cur = self.dbconn.cursor()
					cur.execute("UPDATE cncs SET active = %s WHERE id = %s", [ 'false', self.tname[3:] ])
					self.dbconn.commit()
					cur.close()
					self.stop()
					return
			if not self.running:
				break
			tmp_lines = readbuffer.split('\n')
			readbuffer = tmp_lines.pop()
			for lines in tmp_lines:
				line = lines.rstrip()
				timestamp = datetime.now().strftime('%Y%m%d %H%M%S')
				msg_type = None
				input = line.split()
				if len(input)>=1:
					if input[0] == "PING":
						msg_type = 'ping'
						#mess = "PING received: %s, replying with PONG" % (line)
						#self.Logger.debug(mess)
						reply = "PONG %s\r\n" % (input[1])
						sobj.send(reply)
						LogLine = False
					if input[0] == "ERROR":
						msg_type = 'error'
						if input[1] == ":Closing" and line.find('(Ping timeout)')!=-1:
							mess = "Connection close detected: %s" % (line)
							self.Logger.debug(mess)
							#self.stop()
							if self.reconnect:
								self.init_reconnect = True
							return
				if len(input)>=2:
					if input[1] == "PRIVMSG":
						msg_type = 'priv'
						mess = "Private Message Received: %s" % (line)
						self.Logger.debug(mess)
						# :\x01VERSION\x01
						if input[3].strip() == ":\x01VERSION\x01":
							mess = "Version Request: %s" % (line)
							self.Logger.info(mess)
							reply = "NOTICE %s :\x01VERSION infiltrator\x01\r\n" % (self.nick)
							sobj.send(reply)
							LogLine = False
						else:
							priv_list = input[2:]
							priv_data = " ".join(priv_list)
							self.Logger.debug("Unknown Priv: %s" % (priv_data))
							if not self.priv_messages.has_key(priv_data):
								self.priv_messages[priv_data] = 1
								if self.contains_url(timestamp, priv_data):
									mess = "Possible Download URL Detected: %s" % (line)
									self.Logger.info(mess)
							else:
								self.priv_messages[priv_data] += 1
							self.check_mass_reply(sobj)
							#LogLine = False
					if input[1] == "TOPIC":
						msg_type = 'topic'
						mess = "Topic Message Received: %s" % (line)
						self.Logger.debug(mess)
						topic_list = line[2:]
						topic_data = "".join(topic_list)
						if topic_data not in self.topic:
							self.topic.append(" ".join(topic_list))
						if self.contains_url(timestamp, topic_data):
							mess = "Possible Download URL Detected: %s" % (line)
							self.Logger.info(mess)
						#LogLine = False
				if msg_type != 'ping' and len(line.strip()) > 0:
						cur.execute('INSERT INTO irc_commands (irc_bot, irc_channel, timestamp, sensor, command) VALUES (%s,%s,%s,%s,%s)', [self.bot, self.channel_id, timestamp, self.sensor_name, line])
						self.dbconn.commit()	
				### Ping lines should not be logged
				if LogLine:
					self.Logger.info(line)
					if not self.last_list.full():
						self.last_list.put(line)
					else:
						self.last_list.get()
						self.last_list.put(line)
				else:
					LogLine = True

	def contains_url(self, timestamp, line):
		match = self.url_expre.search(line)
		if match:
			url = match.groups()[0]
			pos = url.rfind('.')
			extension = url[pos:]
#			if extension=='.exe' or extension=='.bat' or extension=='.zip':
			# TODO send malware to db
			self.download_malware(timestamp, url)
#			else:
#				mess = "found URL to none executable: %s" % (url)
#				self.Logger.info(mess)

	def add_sample(self, cur, timestamp, sensor, md5, sample):
		query = 'SELECT md5 FROM samples WHERE md5 = %s'
		cur.execute(query, [ md5 ])
		if cur.rowcount == 0:
			query = 'INSERT INTO samples (md5, sample, size) VALUES (%s, %s, %s)'
			cur.execute(query, [ md5, psycopg2.Binary(sample), len(sample) ])
		query = 'INSERT INTO sightings (md5,experiment,sensor) VALUES (%s, %s, %s)'
		cur.execute(query, [ md5, timestamp, sensor ])

	def download_malware(self, timestamp, url):
		try:
			mess = "Downloading URL: %s" % (url)
			self.Logger.info(mess)
			tmp_pointer = urllib.urlretrieve(url)
			tmp_file = tmp_pointer[0]
		except:
			mess = "URL Content Too Short"
			self.Logger.info(mess)
			return
		if os.path.exists(tmp_file):
			fp = open(tmp_file, 'r')
			content = "".join(fp.readlines())
			fp.close()
			try:
				os.remove(tmp_file)
			except os.error, e:
				mess = "Removing TMP File failed: %s" % (e)
				self.Logger.info(mess)
			hash = md5.new(content)
			fname = hash.hexdigest()
			cur = self.dbconn.cursor()
			self.add_sample(cur, timestamp, self.sensor_name, fname, content)
			self.dbconn.commit()
			cur.close()
			filename = "binaries/%s_%s" % (self.tname, fname)
			if not os.path.exists(filename):
				fp = open(filename, 'a+')
				fp.write(content)
				fp.close()
				mess = "New Binary saved: %s_%s" % (self.tname, fname)
				self.Logger.info(mess)
		else:
			mess = "TMP File is missing"
			self.Logger.info(mess)
			return

	def check_mass_reply(self, sobj):
		priv_keys = self.priv_messages.keys()
		for key in priv_keys:
			number = self.priv_messages[key]
			if number > 10 and self.talkback:
				mess = "Sending Mass PRIVMSG: %s" % (key)
				self.Logger.info(mess)
				reply = "PRIVMSG %s\r\n" % (key)
				sobj.send(reply)
				del self.priv_messages[key]
