#!/bin/bash

VDIBASE=/home/myuser/.VirtualBox/VDI
CURRENT_IMAGE="$1.vdi"
CLEAN_IMAGE="$1-clean.vdi"
VDITOOL=/home/myuser/vditool
CLEAN_MOUNT=/tmp/clean.$$
CURRENT_MOUNT=/tmp/current.$$
TEMPDIFF=/tmp/diff.$$
TODAY="`date +%Y%m%d-%H%M`"
DIFFLOG="/home/myuser/logs/$1-$TODAY.log"
STORE="/home/myuser/store/$1/$TODAY"
SUBMIT="/usr/bin/python /home/myuser/submit.py"
MOUNTVDI="/home/myuser/mountvdi.sh"

VBOXMANAGE=`which vboxmanage`
if [ -z $VBOXMANAGE ]; then
	VBOXMANAGE=`which VBoxManage`
	if [ -z $VBOXMANAGE ]; then
		echo "Cannot find VBoxManage, exiting."
		exit 1
	fi
fi

# mountvdi <vdi> <mountpoint>
function mountvdi {
	local OFFSET=`$VDITOOL DUMP $1 | grep offData | sed 's/^.*=//'`
	sudo mount -t ntfs -o loop,ro,noatime,noexec,offset=$[OFFSET+32256] $1 $2
	rm -f *vditool*.log
}

# clonevdi <vdi> <vdi2>
function clonevdi {
	$VBOXMANAGE -nologo clonevdi $1 $2
}

if [ ! -r "$VDIBASE/$CLEAN_IMAGE" ]; then
	echo "ERROR: cannot find clean image '$VDIBASE/$CLEAN_IMAGE' for VM '$1', exiting."
	exit 1
fi

MY_CURRENT_IMAGE=`$VBOXMANAGE -nologo showvminfo $1|grep 'Primary master' | awk '{ print $3 }'`
if [ "$VDIBASE/$CURRENT_IMAGE" != "$MY_CURRENT_IMAGE" ]; then
	echo "ERROR: current image for VM '$1' is '$MY_CURRENT_IMAGE', but I was expecting '$VDIBASE/$CURRENT_IMAGE', exiting."
	exit 1
fi

$VBOXMANAGE -nologo controlvm $1 poweroff
sleep 5
$VBOXMANAGE -nologo modifyvm $1 -hda none
$VBOXMANAGE -nologo unregisterimage disk $CURRENT_IMAGE

for i in `sudo /sbin/losetup -a|cut -f1 -d:`; do
	sudo /sbin/losetup -d $i
done

mkdir -p $CLEAN_MOUNT $CURRENT_MOUNT
mountvdi "$VDIBASE/$CURRENT_IMAGE" "$CURRENT_MOUNT"
mountvdi "$VDIBASE/$CLEAN_IMAGE" "$CLEAN_MOUNT"

echo "Processing diff log for $1 at `date`" > $DIFFLOG
LANG=C diff -rq $CLEAN_MOUNT $CURRENT_MOUNT > $TEMPDIFF
echo >> $DIFFLOG
echo "Added:" >> $DIFFLOG
grep "^Only in $CURRENT_MOUNT" $TEMPDIFF |sed 's/^Only\ in\ /\"/'|sed 's/\:\ /\//'|sed 's/$/\"/' | xargs ls -ld --full-time >> $DIFFLOG
echo >> $DIFFLOG
echo "Removed:" >> $DIFFLOG
grep "^Only in $CLEAN_MOUNT" $TEMPDIFF |sed 's/^Only\ in\ /\"/'|sed 's/\:\ /\//'|sed 's/$/\"/' | xargs ls -ld --full-time >> $DIFFLOG 
echo >> $DIFFLOG
echo "Changed:" >> $DIFFLOG
grep '^Files.*differ$' $TEMPDIFF | sed 's/Files.*\ and\ \//\"\//'| sed 's/\ differ/\"/' | xargs ls -ld --full-time >> $DIFFLOG
echo >> $DIFFLOG
echo "Processing ended at `date`" >> $DIFFLOG

EXPERIMENT=$TODAY
SENSOR=$1

# warning: ugly hack follows...
mkdir -p $STORE
IFS='|'
LIST=`grep "^Only in $CURRENT_MOUNT" $TEMPDIFF |sed 's/^Only\ in\ //'|sed 's/\:\ /\//'|sed 's/$/|/'|grep exe`
for i in $LIST; do
 	cp -a  `echo $i | tr -d '\n'` $STORE
	FILE=`echo $i | tr -d '\n'`
	unset IFS
	FILENAME=`basename "$FILE"`
	FILESIZE=`ls -l "$FILE" | awk '{ print $5 }'`
	if [ "$FILESIZE" == '0' ]; then
		continue
	fi
	FILEDATE=`ls --full-time "$FILE" | awk '{ print $6 " " $7 " " $8 }'`
	# hack: works only for /tmp/current.$$
	FILEPATH=`dirname "$FILE"| sed 's@/tmp/current\.[0-9]*@@'`
	$SUBMIT -e "$EXPERIMENT" -s "$SENSOR" -n "$FILENAME" -p "$FILEPATH" -d "$FILEDATE" -z "$FILESIZE" "$FILE"
	IFS='|'
done
LIST=`grep '^Files.*differ$' $TEMPDIFF | sed 's/Files.*\ and\ \//\//'| sed 's/\ differ//'|sed 's/$/|/'|grep exe`
for i in $LIST; do
	cp -a `echo $i | tr -d '\n'` $STORE
	FILE=`echo $i | tr -d '\n'`
	unset IFS
	FILENAME=`basename "$FILE"`
	FILESIZE=`ls -l "$FILE" | awk '{ print $5 }'`
	if [ "$FILESIZE" == '0' ]; then
		continue;
	fi
	FILEDATE=`ls --full-time "$FILE" | awk '{ print $6 " " $7 " " $8 }'`
	FILEPATH=`dirname "$FILE"| sed 's@/tmp/current\.[0-9]*@@'`
	$SUBMIT -e "$EXPERIMENT" -s "$SENSOR" -n "$FILENAME" -p "$FILEPATH" -d "$FILEDATE" -z "$FILESIZE" "$FILE"
	IFS='|'
done
unset IFS

rm -f $TEMPDIFF

sudo umount $CURRENT_MOUNT
sudo umount $CLEAN_MOUNT
rmdir $CURRENT_MOUNT $CLEAN_MOUNT

# switch back to the clean image
mv $VDIBASE/$CURRENT_IMAGE $VDIBASE/`basename $CURRENT_IMAGE .vdi`.old.vdi
clonevdi $CLEAN_IMAGE $CURRENT_IMAGE
$VBOXMANAGE -nologo registerimage disk $CURRENT_IMAGE
$VBOXMANAGE -nologo modifyvm $1 -hda $CURRENT_IMAGE
$VBOXMANAGE -nologo startvm $1
