--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

SET search_path = public, pg_catalog;

--
-- Name: md5; Type: DOMAIN; Schema: public; Owner: malware
--
CREATE DOMAIN md5 AS character(32)
	CONSTRAINT md5_check CHECK ((char_length(VALUE) = 32));

--
-- Name: DOMAIN md5; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON DOMAIN md5 IS 'MD5 hash in hex format';


SET default_tablespace = '';
SET default_with_oids = false;

--
-- Name: av_reports; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE av_reports (
    id integer NOT NULL,
    md5 md5 NOT NULL,
    scanner character varying(20) NOT NULL,
    scanner_version character varying(15),
    signature_version date,
    classification character varying(100) NOT NULL,
    processed boolean DEFAULT false NOT NULL
);

--
-- Name: av_accuracy; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW av_accuracy AS
    SELECT a.scanner, (((a.ok_samples)::double precision / (b.scanned_samples)::double precision) * (100)::double precision) AS false_negative_percentage FROM ((SELECT av_reports.scanner, count(av_reports.md5) AS ok_samples FROM av_reports WHERE ((av_reports.classification)::text = 'OK'::text) GROUP BY av_reports.scanner) a NATURAL JOIN (SELECT av_reports.scanner, count(av_reports.md5) AS scanned_samples FROM av_reports GROUP BY av_reports.scanner) b);

--
-- Name: av_class; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW av_class AS
    SELECT sub.classification, sum(sub.ahnlabv3) AS ahnlabv3, sum(sub.antivir) AS antivir, sum(sub.authentium) AS authentium, sum(sub.avast) AS avast, sum(sub.avg) AS avg, sum(sub.bitdefender) AS bitdefender, sum(sub.catquickheal) AS catquickheal, sum(sub.clamav) AS clamav, sum(sub.drweb) AS drweb, sum(sub.esafe) AS esafe, sum(sub.etrustvet) AS etrustvet, sum(sub.ewido) AS ewido, sum(sub.fileadvisor) AS fileadvisor, sum(sub.fortinet) AS fortinet, sum(sub.fprot) AS fprot, sum(sub.fsecure) AS fsecure, sum(sub.gdata) AS gdata, sum(sub.ikarus) AS ikarus, sum(sub.kaspersky) AS kaspersky, sum(sub.mcafee) AS mcafee, sum(sub.microsoft) AS microsoft, sum(sub.nod32v2) AS nod32v2, sum(sub.norman) AS norman, sum(sub.panda) AS panda, sum(sub.prevx1) AS prevx1, sum(sub.rising) AS rising, sum(sub.sophos) AS sophos, sum(sub.sunbelt) AS sunbelt, sum(sub.symantec) AS symantec, sum(sub.thehacker) AS thehacker, sum(sub.vba32) AS vba32, sum(sub.virusbuster) AS virusbuster, sum(sub.webwashergateway) AS webwashergateway FROM (SELECT CASE WHEN (((((((lower((av_reports.classification)::text) ~~ '%allaple%'::text) OR (lower((av_reports.classification)::text) ~~ '%rahack%'::text)) OR (lower((av_reports.classification)::text) ~~ '%endom%'::text)) OR (lower((av_reports.classification)::text) ~~ '%mallar%'::text)) OR (lower((av_reports.classification)::text) ~~ '%starman%'::text)) OR ((av_reports.classification)::text ~~ '%NetApple%'::text)) OR ((av_reports.classification)::text ~~ '%Crypt.XPACK.Gen%'::text)) THEN 'Allaple variant'::character varying WHEN ((((lower((av_reports.classification)::text) ~~ '%virut%'::text) OR (lower((av_reports.classification)::text) ~~ '%virtob%'::text)) OR (lower((av_reports.classification)::text) ~~ '%vetor%'::text)) OR (lower((av_reports.classification)::text) ~~ '%metacrypt%'::text)) THEN 'Virut variant'::character varying WHEN ((lower((av_reports.classification)::text) ~~ '%bacalid%'::text) OR (lower((av_reports.classification)::text) ~~ '%vbac%'::text)) THEN 'Bacalid variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%sdbot%'::text) THEN 'SDBot variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%ircbot%'::text) THEN 'IRCBot variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%vanbot%'::text) THEN 'VanBot variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%cholera%'::text) THEN 'Cholera variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%rbot%'::text) THEN 'RBot variant'::character varying WHEN (lower((av_reports.classification)::text) ~~ '%bobax%'::text) THEN 'Bobax variant'::character varying WHEN ((lower((av_reports.classification)::text) ~~ '%eggdrop%'::text) OR ((av_reports.classification)::text = 'Exploit:Win32/MS06040.gen'::text)) THEN 'eggdrop variant'::character varying WHEN ((((lower((av_reports.classification)::text) ~~ '%malicious%'::text) OR (lower((av_reports.classification)::text) ~~ '%cloaked%'::text)) OR (lower((av_reports.classification)::text) ~~ '%fileinfector%'::text)) OR (lower((av_reports.classification)::text) ~~ '%emailworm%'::text)) THEN 'Generic malware'::character varying WHEN ((((lower((av_reports.classification)::text) ~~ '%suspicious%'::text) OR (lower((av_reports.classification)::text) ~~ '%suspected%'::text)) OR (lower((av_reports.classification)::text) ~~ '%probably%'::text)) OR (lower((av_reports.classification)::text) ~~ '%heuristic%'::text)) THEN 'Suspicious file'::character varying ELSE av_reports.classification END AS classification, sum(CASE WHEN ((av_reports.scanner)::text = 'AhnLab-V3'::text) THEN 1 ELSE 0 END) AS ahnlabv3, sum(CASE WHEN ((av_reports.scanner)::text = 'AntiVir'::text) THEN 1 ELSE 0 END) AS antivir, sum(CASE WHEN ((av_reports.scanner)::text = 'Authentium'::text) THEN 1 ELSE 0 END) AS authentium, sum(CASE WHEN ((av_reports.scanner)::text = 'Avast'::text) THEN 1 ELSE 0 END) AS avast, sum(CASE WHEN ((av_reports.scanner)::text = 'AVG'::text) THEN 1 ELSE 0 END) AS avg, sum(CASE WHEN ((av_reports.scanner)::text = 'BitDefender'::text) THEN 1 ELSE 0 END) AS bitdefender, sum(CASE WHEN ((av_reports.scanner)::text = 'CAT-QuickHeal'::text) THEN 1 ELSE 0 END) AS catquickheal, sum(CASE WHEN ((av_reports.scanner)::text = 'ClamAV'::text) THEN 1 ELSE 0 END) AS clamav, sum(CASE WHEN ((av_reports.scanner)::text = 'DrWeb'::text) THEN 1 ELSE 0 END) AS drweb, sum(CASE WHEN ((av_reports.scanner)::text = 'eSafe'::text) THEN 1 ELSE 0 END) AS esafe, sum(CASE WHEN ((av_reports.scanner)::text = 'eTrust-Vet'::text) THEN 1 ELSE 0 END) AS etrustvet, sum(CASE WHEN ((av_reports.scanner)::text = 'Ewido'::text) THEN 1 ELSE 0 END) AS ewido, sum(CASE WHEN ((av_reports.scanner)::text = 'FileAdvisor'::text) THEN 1 ELSE 0 END) AS fileadvisor, sum(CASE WHEN ((av_reports.scanner)::text = 'Fortinet'::text) THEN 1 ELSE 0 END) AS fortinet, sum(CASE WHEN ((av_reports.scanner)::text = 'F-Prot'::text) THEN 1 ELSE 0 END) AS fprot, sum(CASE WHEN ((av_reports.scanner)::text = 'F-Secure'::text) THEN 1 ELSE 0 END) AS fsecure, sum(CASE WHEN ((av_reports.scanner)::text = 'GData'::text) THEN 1 ELSE 0 END) AS gdata, sum(CASE WHEN ((av_reports.scanner)::text = 'Ikarus'::text) THEN 1 ELSE 0 END) AS ikarus, sum(CASE WHEN ((av_reports.scanner)::text = 'Kaspersky'::text) THEN 1 ELSE 0 END) AS kaspersky, sum(CASE WHEN ((av_reports.scanner)::text = 'McAfee'::text) THEN 1 ELSE 0 END) AS mcafee, sum(CASE WHEN ((av_reports.scanner)::text = 'Microsoft'::text) THEN 1 ELSE 0 END) AS microsoft, sum(CASE WHEN ((av_reports.scanner)::text = 'NOD32v2'::text) THEN 1 ELSE 0 END) AS nod32v2, sum(CASE WHEN ((av_reports.scanner)::text = 'Norman'::text) THEN 1 ELSE 0 END) AS norman, sum(CASE WHEN ((av_reports.scanner)::text = 'Panda'::text) THEN 1 ELSE 0 END) AS panda, sum(CASE WHEN ((av_reports.scanner)::text = 'Prevx1'::text) THEN 1 ELSE 0 END) AS prevx1, sum(CASE WHEN ((av_reports.scanner)::text = 'Rising'::text) THEN 1 ELSE 0 END) AS rising, sum(CASE WHEN ((av_reports.scanner)::text = 'Sophos'::text) THEN 1 ELSE 0 END) AS sophos, sum(CASE WHEN ((av_reports.scanner)::text = 'Sunbelt'::text) THEN 1 ELSE 0 END) AS sunbelt, sum(CASE WHEN ((av_reports.scanner)::text = 'Symantec'::text) THEN 1 ELSE 0 END) AS symantec, sum(CASE WHEN ((av_reports.scanner)::text = 'TheHacker'::text) THEN 1 ELSE 0 END) AS thehacker, sum(CASE WHEN ((av_reports.scanner)::text = 'VBA32'::text) THEN 1 ELSE 0 END) AS vba32, sum(CASE WHEN ((av_reports.scanner)::text = 'VirusBuster'::text) THEN 1 ELSE 0 END) AS virusbuster, sum(CASE WHEN ((av_reports.scanner)::text = 'Webwasher-Gateway'::text) THEN 1 ELSE 0 END) AS webwashergateway FROM av_reports GROUP BY av_reports.classification) sub WHERE (((((((((((((((((((((((((((((((((sub.ahnlabv3 > 10) OR (sub.antivir > 10)) OR (sub.authentium > 10)) OR (sub.avast > 10)) OR (sub.avg > 10)) OR (sub.bitdefender > 10)) OR (sub.catquickheal > 10)) OR (sub.clamav > 10)) OR (sub.drweb > 10)) OR (sub.esafe > 10)) OR (sub.etrustvet > 10)) OR (sub.ewido > 10)) OR (sub.fileadvisor > 10)) OR (sub.fortinet > 10)) OR (sub.fprot > 10)) OR (sub.fsecure > 10)) OR (sub.gdata > 10)) OR (sub.ikarus > 10)) OR (sub.kaspersky > 10)) OR (sub.mcafee > 10)) OR (sub.microsoft > 10)) OR (sub.nod32v2 > 10)) OR (sub.norman > 10)) OR (sub.panda > 10)) OR (sub.prevx1 > 10)) OR (sub.rising > 10)) OR (sub.sophos > 10)) OR (sub.sunbelt > 10)) OR (sub.symantec > 10)) OR (sub.thehacker > 10)) OR (sub.vba32 > 10)) OR (sub.virusbuster > 10)) OR (sub.webwashergateway > 10)) GROUP BY sub.classification ORDER BY sum(sub.antivir) DESC;

--
-- Name: cnc_type; Type: TYPE; Schema: public; Owner: malware
--

CREATE TYPE cnc_type AS ENUM (
    'irc',
    'http'
);


ALTER TYPE public.cnc_type OWNER TO malware;

--
-- Name: TYPE cnc_type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON TYPE cnc_type IS 'Type of command center';

--
-- Name: cncs_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE cncs_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: cncs; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE cncs (
    id integer DEFAULT nextval('cncs_id_seq'::regclass) NOT NULL,
    address inet NOT NULL,
    port integer NOT NULL,
    type cnc_type NOT NULL,
    dnsname character varying(50),
    country character(2),
    active boolean DEFAULT true,
    latitude double precision,
    longitude double precision
);

--
-- Name: COLUMN cncs.type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN cncs.type IS 'irc or http';


--
-- Name: COLUMN cncs.dnsname; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN cncs.dnsname IS 'DNS name as requested by the malware';


--
-- Name: COLUMN cncs.country; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN cncs.country IS 'ISO 3166 country code';


--
-- Name: malware_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE malware_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
    
--
-- Name: SEQUENCE malware_id_seq; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON SEQUENCE malware_id_seq IS 'Malware ID; used in worms and *_bots, referenced in samples.';


--
-- Name: http_bots; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE http_bots (
    id integer DEFAULT nextval('malware_id_seq'::regclass) NOT NULL,
    cnc integer NOT NULL,
    last_seen timestamp without time zone
);

--
-- Name: irc_bots; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_bots (
    id integer DEFAULT nextval('malware_id_seq'::regclass) NOT NULL,
    cnc integer NOT NULL,
    hostname character varying(20),
    realname character varying(400),
    nickname character varying(50),
    username character varying(20),
    password character varying(20),
    rfc_compliant boolean,
    servername character varying(20),
    last_seen timestamp without time zone
);

--
-- Name: cncs_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW cncs_brk AS
    SELECT cncs.id, cncs.address, cncs.port, cncs.dnsname, cncs.type, count(http_bots.id) AS num_bots, max(http_bots.last_seen) AS last_seen, cncs.country, cncs.latitude, cncs.longitude FROM http_bots, cncs WHERE (http_bots.cnc = cncs.id) GROUP BY cncs.id, cncs.address, cncs.port, cncs.dnsname, cncs.type, cncs.country, cncs.latitude, cncs.longitude UNION SELECT cncs.id, cncs.address, cncs.port, cncs.dnsname, cncs.type, count(irc_bots.id) AS num_bots, max(irc_bots.last_seen) AS last_seen, cncs.country, cncs.latitude, cncs.longitude FROM irc_bots, cncs WHERE (irc_bots.cnc = cncs.id) GROUP BY cncs.id, cncs.address, cncs.port, cncs.dnsname, cncs.type, cncs.country, cncs.latitude, cncs.longitude ORDER BY 6 DESC;

--
-- Name: malware_type; Type: TYPE; Schema: public; Owner: malware
--

CREATE TYPE malware_type AS ENUM (
    'worm',
    'irc_bot',
    'http_bot'
);

--
-- Name: TYPE malware_type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON TYPE malware_type IS 'Malware type categorization';


--
-- Name: samples_ref; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE samples_ref (
    sample md5 NOT NULL,
    type malware_type NOT NULL,
    malware_id integer NOT NULL
);

--
-- Name: COLUMN samples_ref.sample; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples_ref.sample IS 'Sample MD5 hash';


--
-- Name: COLUMN samples_ref.type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples_ref.type IS 'Malware type';


--
-- Name: COLUMN samples_ref.malware_id; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples_ref.malware_id IS 'Malware identification for this sample; references to <type>  table''s field ''id''';


--
-- Name: sightings; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE sightings (
    id integer NOT NULL,
    md5 md5 NOT NULL,
    sensor character varying(15) NOT NULL,
    experiment timestamp without time zone,
    filename character varying(255),
    filepath character varying(255),
    filedate timestamp without time zone
);

--
-- Name: daily_samples_per_type; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW daily_samples_per_type AS
    SELECT sub.date, sum(CASE WHEN (sub.type = 'worm'::malware_type) THEN 1 ELSE 0 END) AS worms, sum(CASE WHEN (sub.type = 'irc_bot'::malware_type) THEN 1 ELSE 0 END) AS irc_bots, sum(CASE WHEN (sub.type = 'http_bot'::malware_type) THEN 1 ELSE 0 END) AS http_bots, count(sub.type) AS total FROM (SELECT DISTINCT ON (sightings.md5) (sightings.experiment)::date AS date, samples_ref.type FROM samples_ref, sightings WHERE ((sightings.md5)::bpchar = (samples_ref.sample)::bpchar) ORDER BY sightings.md5) sub GROUP BY sub.date ORDER BY sub.date;

--
-- Name: daily_sightings_per_type; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW daily_sightings_per_type AS
    SELECT (sightings.experiment)::date AS date, sum(CASE WHEN (samples_ref.type = 'worm'::malware_type) THEN 1 ELSE 0 END) AS worms, sum(CASE WHEN (samples_ref.type = 'irc_bot'::malware_type) THEN 1 ELSE 0 END) AS irc_bots, sum(CASE WHEN (samples_ref.type = 'http_bot'::malware_type) THEN 1 ELSE 0 END) AS http_bots, count(samples_ref.type) AS total FROM sightings, samples_ref WHERE ((sightings.md5)::bpchar = (samples_ref.sample)::bpchar) GROUP BY (sightings.experiment)::date ORDER BY (sightings.experiment)::date;

--
-- Name: http_bots_ref; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE http_bots_ref (
    http_bot integer NOT NULL,
    http_cmd integer NOT NULL
);

--
-- Name: http_cmds; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE http_cmds (
    id integer NOT NULL,
    url character varying(500) NOT NULL,
    method character varying(5) NOT NULL,
    version character varying(10) NOT NULL,
    headers text
);

--
-- Name: http_replies; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE http_replies (
    id integer NOT NULL,
    "timestamp" timestamp without time zone NOT NULL,
    http_bot integer NOT NULL,
    http_cmd integer NOT NULL,
    status smallint NOT NULL,
    reason character varying(30) NOT NULL,
    headers text,
    response bytea,
    sensor character varying(10)
);

--
-- Name: irc_bots_refc; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_bots_refc (
    irc_bot integer NOT NULL,
    irc_channel integer NOT NULL
);

--
-- Name: irc_bots_refp; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_bots_refp (
    irc_bot integer NOT NULL,
    irc_privmsg integer NOT NULL
);

--
-- Name: samples; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE samples (
    md5 md5 NOT NULL,
    type character varying(10),
    size integer,
    malware_id integer,
    sample bytea,
    CONSTRAINT md5_length CHECK ((char_length((md5)::bpchar) = 32)),
    CONSTRAINT size_check CHECK ((size > 0))
);

--
-- Name: COLUMN samples.md5; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples.md5 IS 'MD5 hash of the collected sample';


--
-- Name: COLUMN samples.type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples.type IS 'DEPRECATED';


--
-- Name: COLUMN samples.size; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples.size IS 'Sample size in bytes';


--
-- Name: COLUMN samples.malware_id; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples.malware_id IS 'DEPRECATED';


--
-- Name: COLUMN samples.sample; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN samples.sample IS 'The collected sample binary file';


--
-- Name: irc_bots_viruses; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW irc_bots_viruses AS
    SELECT samples.md5, samples_ref.malware_id, cncs.address, cncs.dnsname, av_reports.classification FROM samples, samples_ref, av_reports, irc_bots, cncs WHERE ((((((samples.md5)::bpchar = (samples_ref.sample)::bpchar) AND (samples_ref.malware_id = irc_bots.id)) AND ((samples.md5)::bpchar = (av_reports.md5)::bpchar)) AND ((av_reports.scanner)::text = 'ClamAV-local'::text)) AND (irc_bots.cnc = cncs.id)) ORDER BY av_reports.classification;

--
-- Name: irc_channels; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_channels (
    id integer NOT NULL,
    name character varying(15) NOT NULL,
    password character varying(15),
    topics character varying(500)
);

--
-- Name: irc_commands; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_commands (
    id integer NOT NULL,
    irc_bot integer NOT NULL,
    irc_channel integer,
    "timestamp" timestamp without time zone NOT NULL,
    command text NOT NULL,
    sensor character varying(15)
);

--
-- Name: irc_commands_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW irc_commands_brk AS
    SELECT sum(CASE WHEN (irc_commands.command ~~ '%PRIVMSG%'::text) THEN 1 ELSE 0 END) AS priv, sum(CASE WHEN (irc_commands.command ~~ '%NOTICE%'::text) THEN 1 ELSE 0 END) AS notice, sum(CASE WHEN (irc_commands.command ~~ '%QUIT%'::text) THEN 1 ELSE 0 END) AS quit, sum(CASE WHEN (irc_commands.command ~~ '%JOIN%'::text) THEN 1 ELSE 0 END) AS "join", sum(CASE WHEN (irc_commands.command ~~ '%MODE%'::text) THEN 1 ELSE 0 END) AS mode, sum(CASE WHEN (irc_commands.command ~~ '%TOPIC%'::text) THEN 1 ELSE 0 END) AS topic, sum(CASE WHEN ((lower(irc_commands.command) ~~ '%scan%'::text) OR (lower(irc_commands.command) ~~ '%.asc asn%'::text)) THEN 1 ELSE 0 END) AS bot_scan, sum(CASE WHEN (lower(irc_commands.command) ~~ '%!login%'::text) THEN 1 ELSE 0 END) AS bot_login, sum(CASE WHEN ((((lower(irc_commands.command) ~~ '%!get%'::text) OR (lower(irc_commands.command) ~~ '%.get%'::text)) OR (lower(irc_commands.command) ~~ '%*get%'::text)) OR (lower(irc_commands.command) ~~ '%!nokia%'::text)) THEN 1 ELSE 0 END) AS bot_update, sum(CASE WHEN (lower(irc_commands.command) ~~ '%*pst %'::text) THEN 1 ELSE 0 END) AS data_mining FROM irc_commands;

--
-- Name: irc_privmsgs; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE irc_privmsgs (
    id integer NOT NULL,
    type character(1) NOT NULL,
    message character varying(500) NOT NULL
);

--
-- Name: COLUMN irc_privmsgs.type; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN irc_privmsgs.type IS 'Notice or Privmsg';


--
-- Name: path_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW path_brk AS
    SELECT sightings.filepath, count(sightings.filepath) AS num_sightings FROM sightings GROUP BY sightings.filepath ORDER BY count(sightings.filepath) DESC;

--
-- Name: provider; Type: TYPE; Schema: public; Owner: malware
--

CREATE TYPE provider AS ENUM (
    'Anubis',
    'CWSandbox',
    'VirusTotal'
);

--
-- Name: TYPE provider; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON TYPE provider IS 'Analysis report provider';


--
-- Name: reports; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE reports (
    id integer NOT NULL,
    md5 md5 NOT NULL,
    provider provider NOT NULL,
    available boolean DEFAULT false,
    task_id character(32),
    analysis_id integer,
    password character(5),
    processed boolean DEFAULT false,
    report xml,
    pcap bytea,
    CONSTRAINT reports_password CHECK ((char_length(password) = 5))
);

--
-- Name: COLUMN reports.report; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN reports.report IS 'Report data in XML format (if available)';


--
-- Name: COLUMN reports.pcap; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON COLUMN reports.pcap IS 'PCAP network trace (if available)';


--
-- Name: samples_per_day; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW samples_per_day AS
    SELECT sum(pippo2.num_samples) AS num_samples, pippo2.experiment FROM (SELECT count(pippo.md5) AS num_samples, (pippo.experiment)::date AS experiment FROM (SELECT DISTINCT ON (samples.md5) samples.md5, sightings.experiment FROM samples, sightings WHERE ((samples.md5)::bpchar = (sightings.md5)::bpchar) ORDER BY samples.md5) pippo GROUP BY pippo.experiment) pippo2 GROUP BY pippo2.experiment ORDER BY pippo2.experiment;

--
-- Name: sensors; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE sensors (
    name character varying(15) NOT NULL,
    os character varying(20),
    ip inet,
    low_interaction boolean DEFAULT false,
    notes character varying(200)
);

--
-- Name: sensors_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW sensors_brk AS
    SELECT sightings.sensor, count(*) AS num_sightings FROM sightings GROUP BY sightings.sensor ORDER BY count(*) DESC;

--
-- Name: settings; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE settings (
    key character varying(20) NOT NULL,
    value text NOT NULL
);

--
-- Data for Name: settings; Type: TABLE DATA; Schema: public; Owner: davide
--

INSERT INTO settings VALUES ('PYTHON_EXTRA_PATH', '/home/malware');
INSERT INTO settings VALUES ('SAMPLES_PATH', '/var/www/uploads');
INSERT INTO settings VALUES ('SAMPLES_GROUP', 'www-data');
INSERT INTO settings VALUES ('CWSANDBOX_XML_URL', 'http://www.cwsandbox.org/?page=analysis&id=$ANALYSIS_ID$&format=xml&password=$PASSWORD$');
INSERT INTO settings VALUES ('CWSANDBOX_PCAP_URL', 'http://www.cwsandbox.org/?page=download&dltype=pcap&id=$ANALYSIS_ID$&password=$PASSWORD$');
INSERT INTO settings VALUES ('GEOIP_DB', '/home/malware/GeoLiteCity.dat');
INSERT INTO settings VALUES ('REPORTS_EMAIL_ADDR', 'malware@example.com');
INSERT INTO settings VALUES ('REPORTS_EMAIL_USER', 'malware@example.com');
INSERT INTO settings VALUES ('REPORTS_EMAIL_PASS', 'email_password');
INSERT INTO settings VALUES ('CWSANDBOX_SAMDET_URL', 'http://www.cwsandbox.org/?page=samdet&id=$SAMPLE_ID$&password=$PASSWORD$');
INSERT INTO settings VALUES ('ANUBIS_REPORT_URL', 'http://analysis.seclab.tuwien.ac.at/result.php?taskid=$TASK_ID$');
INSERT INTO settings VALUES ('ANUBIS_SUBMIT_URL', 'http://analysis.seclab.tuwien.ac.at/web_action.php');
INSERT INTO settings VALUES ('CWSANDBOX_SUBMIT_URL', 'http://www.cwsandbox.org/?page=submit&action=verify');

--
-- Name: sightings_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW sightings_brk AS
    SELECT sightings.md5, count(*) AS num_sightings FROM samples, sightings WHERE ((samples.md5)::bpchar = (sightings.md5)::bpchar) GROUP BY sightings.md5 ORDER BY count(*) DESC;

--
-- Name: sightings_per_day; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW sightings_per_day AS
    SELECT sum(pippo2.num_sightings) AS num_sightings, pippo2.experiment FROM (SELECT count(sightings.md5) AS num_sightings, (sightings.experiment)::date AS experiment FROM sightings GROUP BY sightings.experiment) pippo2 GROUP BY pippo2.experiment ORDER BY pippo2.experiment;

--
-- Name: size_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW size_brk AS
    SELECT samples.size, count(samples.size) AS num_samples FROM samples GROUP BY samples.size ORDER BY count(samples.size) DESC;

--
-- Name: targets; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE targets (
    id integer NOT NULL,
    address inet NOT NULL,
    port integer,
    protocol character varying(5) NOT NULL,
    attacker integer NOT NULL,
    last_seen timestamp without time zone NOT NULL,
    dnsname character varying(50),
    established boolean
);

--
-- Name: worms; Type: TABLE; Schema: public; Owner: malware; Tablespace: 
--

CREATE TABLE worms (
    id integer DEFAULT nextval('malware_id_seq'::regclass) NOT NULL,
    name character varying(10) NOT NULL
);

--
-- Name: type_brk; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW type_brk AS
    ((SELECT '(unfiled)' AS type, count(samples.md5) AS num_samples, NULL::unknown AS num_unique FROM samples WHERE (NOT ((samples.md5)::bpchar IN (SELECT DISTINCT samples_ref.sample FROM samples_ref ORDER BY samples_ref.sample))) UNION SELECT (samples_ref.type)::text AS type, count(samples_ref.sample) AS num_samples, count(DISTINCT irc_bots.cnc) AS num_unique FROM samples_ref, irc_bots WHERE (samples_ref.malware_id = irc_bots.id) GROUP BY samples_ref.type) UNION SELECT (samples_ref.type)::text AS type, count(samples_ref.sample) AS num_samples, count(DISTINCT http_bots.cnc) AS num_unique FROM samples_ref, http_bots WHERE (samples_ref.malware_id = http_bots.id) GROUP BY samples_ref.type) UNION SELECT (samples_ref.type)::text AS type, count(samples_ref.sample) AS num_samples, count(DISTINCT worms.id) AS num_unique FROM samples_ref, worms WHERE (samples_ref.malware_id = worms.id) GROUP BY samples_ref.type;

--
-- Name: weekly_samples_per_type; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW weekly_samples_per_type AS
    SELECT date_part('week'::text, daily_samples_per_type.date) AS week, sum(daily_samples_per_type.worms) AS worms, sum(daily_samples_per_type.irc_bots) AS irc_bots, sum(daily_samples_per_type.http_bots) AS http_bots, sum(daily_samples_per_type.total) AS total FROM daily_samples_per_type GROUP BY date_part('week'::text, daily_samples_per_type.date) ORDER BY date_part('week'::text, daily_samples_per_type.date);

--
-- Name: weekly_sightings_per_type; Type: VIEW; Schema: public; Owner: malware
--

CREATE VIEW weekly_sightings_per_type AS
    SELECT date_part('week'::text, daily_sightings_per_type.date) AS week, sum(daily_sightings_per_type.worms) AS worms, sum(daily_sightings_per_type.irc_bots) AS irc_bots, sum(daily_sightings_per_type.http_bots) AS http_bots, sum(daily_sightings_per_type.total) AS total FROM daily_sightings_per_type GROUP BY date_part('week'::text, daily_sightings_per_type.date) ORDER BY date_part('week'::text, daily_sightings_per_type.date);

--
-- Name: add_cnc(inet, integer, cnc_type, character varying); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_cnc(myaddress inet, myport integer, mytype cnc_type, mydnsname character varying) RETURNS integer
    AS $$
DECLARE
       val integer;
       dns varchar;
BEGIN
     SELECT id, dnsname INTO val, dns FROM cncs 
            WHERE address = myaddress and port = myport and type = mytype;
     IF val IS NULL THEN
          INSERT INTO cncs (id, address, port, type, dnsname) 
                 VALUES (DEFAULT, myaddress, myport, mytype, mydnsname) 
                 RETURNING id INTO val;
     ELSEIF dns IS NULL AND mydnsname IS NOT NULL THEN
          UPDATE cncs SET dnsname = mydnsname WHERE id = val;
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: add_http_bot(integer, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_http_bot(mycnc integer, mydate timestamp without time zone) RETURNS integer
    AS $$
DECLARE
       val integer;
       olddate timestamp;
BEGIN
     SELECT id, last_seen INTO val, olddate FROM http_bots WHERE cnc =  mycnc;
     IF val IS NULL THEN
        INSERT INTO http_bots (id, cnc, last_seen) 
               VALUES (DEFAULT, mycnc, mydate) RETURNING id INTO val;
     ELSEIF mydate > olddate THEN
        UPDATE http_bots SET last_seen = mydate WHERE id = val;
     END IF;
     RETURN val;   
END;
$$
    LANGUAGE plpgsql;

--
-- Name: add_http_cmd(integer, character varying, character varying, character varying, text); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_http_cmd(mybot integer, myurl character varying, mymethod character varying, myversion character varying, myheaders text) RETURNS integer
    AS $$
DECLARE
       val integer;
       temprow http_bots_ref%ROWTYPE;
BEGIN
     SELECT id INTO val FROM http_cmds WHERE 
            url = myurl AND method = mymethod AND 
            version = myversion AND headers = myheaders;
     IF val IS NULL THEN
        INSERT INTO http_cmds (id, url, method, version, headers) 
            VALUES (DEFAULT, myurl, mymethod, myversion, myheaders)
            RETURNING id INTO val;
     END IF;
     SELECT * INTO temprow FROM http_bots_ref WHERE http_bot = mybot AND http_cmd = val;
     IF NOT FOUND THEN
          INSERT INTO http_bots_ref (http_bot, http_cmd) VALUES (mybot, val);
     ELSE
          RAISE NOTICE 'Command % is already linked to bot %, skipping', val, mybot;
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: FUNCTION add_http_cmd(mybot integer, myurl character varying, mymethod character varying, myversion character varying, myheaders text); Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON FUNCTION add_http_cmd(mybot integer, myurl character varying, mymethod character varying, myversion character varying, myheaders text) IS 'Add a new HTTP command and link it with the provided bot. If an identical command already exists only establish the link. If an identical link already exists raise a notice and abort.';


--
-- Name: add_irc_bot(integer, character varying, character varying, character varying, character varying, character varying, character varying, boolean, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_irc_bot(mycnc integer, myhostname character varying, myrealname character varying, mynickname character varying, myusername character varying, mypassword character varying, myservername character varying, myrfccomp boolean, mydate timestamp without time zone) RETURNS integer
    AS $$
DECLARE
       val integer;
       olddate timestamp;
BEGIN
     SELECT id, last_seen INTO val, olddate FROM irc_bots WHERE cnc = mycnc AND 
            hostname = myhostname AND realname = myrealname AND
            nickname = mynickname AND username = myusername AND
            password = mypassword AND servername = myservername AND
            rfc_compliant = myrfccomp;
     IF val IS NULL THEN
            INSERT INTO irc_bots 
                   (id,cnc,hostname,realname,nickname,username,password,servername,rfc_compliant,last_seen)
            VALUES (DEFAULT,mycnc,myhostname,myrealname,mynickname,myusername,mypassword,myservername,myrfccomp,mydate)
            RETURNING id INTO val;
     ELSEIF mydate > olddate THEN
            UPDATE irc_bots SET last_seen = mydate WHERE id = val;      
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: add_irc_channel(integer, character varying, character varying, text); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_irc_channel(mybot integer, myname character varying, mypassword character varying, mytopics text) RETURNS integer
    AS $$
DECLARE
       val integer;     
       myrow irc_channels%ROWTYPE;
       temprow irc_bots_refc%ROWTYPE;
BEGIN
     SELECT id INTO val FROM irc_channels WHERE 
            name = myname AND password = mypassword AND topics = mytopics;
     IF val IS NULL THEN
            SELECT * INTO myrow FROM irc_channels WHERE name = myname;
            val := myrow.id;
            IF val IS NULL OR NOT (
               (myrow.name = myname) AND 
               (myrow.password IS NULL OR myrow.password = mypassword) AND 
               (myrow.topics IS NULL OR myrow.topics = mytopics)
            ) THEN
                   INSERT INTO irc_channels (id, name, password, topics)
                          VALUES (DEFAULT, myname, mypassword, mytopics)
                          RETURNING id INTO val;
            END IF;
     END IF;
     SELECT * INTO temprow FROM irc_bots_refc WHERE irc_bot = mybot AND irc_channel = val;
     IF NOT FOUND THEN
          INSERT INTO irc_bots_refc (irc_bot, irc_channel) VALUES (mybot, val);
     ELSE
          RAISE NOTICE 'Channel % is already linked to bot %, skipping', val, mybot;
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: FUNCTION add_irc_channel(mybot integer, myname character varying, mypassword character varying, mytopics text); Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON FUNCTION add_irc_channel(mybot integer, myname character varying, mypassword character varying, mytopics text) IS 'Add a new IRC channel and link it with the provided bot. If an identical channel already exists only establish the link. If an identical link already exists raise a notice and abort.';


--
-- Name: add_irc_privmsg(integer, character varying, character varying); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_irc_privmsg(mybot integer, mytype character varying, mymessage character varying) RETURNS integer
    AS $$
DECLARE
       val integer;
       temprow irc_bots_refp%ROWTYPE;
BEGIN
     SELECT id INTO val FROM irc_privmsgs WHERE 
            type = mytype AND message = mymessage;
     IF val IS NULL THEN
            INSERT INTO irc_privmsgs (id, type, message) VALUES (DEFAULT, mytype, mymessage)
            RETURNING id INTO val;
     END IF;
     SELECT * INTO temprow FROM irc_bots_refp WHERE irc_bot = mybot AND irc_privmsg = val;
     IF NOT FOUND THEN
          INSERT INTO irc_bots_refp (irc_bot, irc_privmsg) VALUES (mybot, val);
     ELSE
          RAISE NOTICE 'Message % is already linked to bot %, skipping', val, mybot;
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: FUNCTION add_irc_privmsg(mybot integer, mytype character varying, mymessage character varying); Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON FUNCTION add_irc_privmsg(mybot integer, mytype character varying, mymessage character varying) IS 'Add a new IRC message and link it with the provided bot. If an identical message already exists only establish the link. If an identical link already exists raise a notice and abort.';


--
-- Name: add_target(inet, integer, character varying, integer, timestamp without time zone, character varying, boolean); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION add_target(myaddress inet, myport integer, myproto character varying, myattacker integer, mydate timestamp without time zone, mydnsname character varying, myestablished boolean) RETURNS integer
    AS $$
DECLARE
       val integer;
       olddate timestamp;
BEGIN
     SELECT id, last_seen INTO val, olddate FROM targets 
            WHERE address = myaddress AND (port = myport OR (port IS NULL AND protocol = 'ICMP'))
                  AND protocol = myproto AND attacker = myattacker AND dnsname = mydnsname;
     IF val IS NULL THEN
        INSERT INTO targets (id, address, port, protocol, attacker, last_seen, dnsname, established) 
               VALUES (DEFAULT, myaddress, myport, myproto, myattacker, mydate, mydnsname, myestablished) 
               RETURNING id INTO val;
     ELSEIF mydate > olddate THEN
        UPDATE targets SET last_seen = mydate WHERE id = val;
     END IF;
     RETURN val;   
END;
$$
    LANGUAGE plpgsql;

--
-- Name: build_missing_reports(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION build_missing_reports() RETURNS boolean
    AS $$
DECLARE 
    myrow RECORD;
BEGIN
    FOR myrow IN 
        SELECT md5 FROM samples EXCEPT 
            (SELECT md5 FROM reports WHERE provider = 'Anubis') 
    LOOP
        RAISE NOTICE 'Submitting sample % to Anubis...', myrow.md5;
        PERFORM submit_to_anubis(myrow.md5);
    END LOOP;
    
    FOR myrow IN 
        SELECT md5 FROM samples EXCEPT
            (SELECT md5 FROM reports WHERE provider = 'CWSandbox' OR provider = 'VirusTotal')
except 
       (select md5 from reports where provider='CWSandbox' or provider='VirusTotal')
    LOOP
        RAISE NOTICE 'Submitting sample % to CWSandbox...', myrow.md5;
        PERFORM submit_to_cwsandbox(myrow.md5);
    END LOOP;
    
    RETURN 'true';
END;
$$
    LANGUAGE plpgsql;

--
-- Name: check_report_availability(integer); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION check_report_availability(id integer) RETURNS boolean
    AS $_$
import urllib
import urlparse
import re

if not SD.has_key('select'):
    SD['select'] = plpy.prepare(
        'SELECT provider, task_id, analysis_id, password FROM reports WHERE id = $1', 
        [ 'text' ]);
res = plpy.execute(SD['select'], [ id ])
if len(res) == 0:
    return 'false'

provider = res[0]['provider']   

if not SD.has_key('results_url'):
    SD['results'] = plpy.prepare('SELECT results_url FROM providers WHERE name = $1', [ 'text'])
r = plpy.execute(SD['results'], [ provider ])
url = r[0]['results_url']

if provider == 'Anubis':
    if not SD.has_key('regexp'):
        SD['regexp'] = re.compile("<title>.*</title>")
    task_id = res[0]['task_id']
    f = urllib.urlopen(url.replace('$TASK_ID$', task_id))
    s = SD['regexp'].search(f.read())
    f.close()
    if (s != None) and (s.group() == '<title>Anubis - Analysis Report</title>'):
        return 'true'
    else:
        return 'false'
elif provider == 'CWSandbox':
    #TODO da fare
    return 'true'
else:
    plpy.warning('Unknown provider "' + provider + '" for report "' + id + '"')
    return 'false'
$_$
    LANGUAGE plpythonu;

--
-- Name: check_sample_file(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION check_sample_file() RETURNS trigger
    AS $$
import os
if TD["event"] == "INSERT" or TD["event"] == "UPDATE":
    if TD["event"] == "UPDATE" and TD["new"]["md5"] == None:
        return "OK"
    md5 = TD["new"]["md5"].rstrip()
    if len(md5) != 32:
        return "SKIP"
    if os.path.exists("/var/www/uploads/" + md5):
        return "OK"
    else:
        return "SKIP"
elif TD["event"] == "DELETE":
    file = "/var/www/uploads/" + TD["old"]["md5"].rstrip()
    if os.path.exists(file):
        os.unlink(file)
    return "OK"
else:
    return "SKIP"
$$
    LANGUAGE plpythonu;

--
-- Name: do_submit(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION do_submit() RETURNS trigger
    AS $$
DECLARE
       val boolean DEFAULT TRUE;
BEGIN
     SELECT submit_to_clamav(NEW.md5) INTO val;
     IF val IS TRUE THEN
        PERFORM submit_to_anubis(NEW.md5);
        PERFORM submit_to_cwsandbox(NEW.md5);
     END IF;
     RETURN NULL; -- ignored, this is run as AFTER trigger
END;
$$
    LANGUAGE plpgsql;

--
-- Name: process_av_report(character); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION process_av_report(mymd5 character) RETURNS boolean
    AS $$
DECLARE
       -- constants
       WORM_TYPE CONSTANT malware_type DEFAULT 'worm';
       ALLAPLE_ID CONSTANT integer DEFAULT 1;
       ALLAPLE_THRESHOLD CONSTANT integer DEFAULT 5;
       -- variables
       myrow av_reports%ROWTYPE;
       classification varchar(100);
       val boolean DEFAULT false;
       allaple integer DEFAULT 0;
BEGIN
     FOR myrow IN SELECT * FROM av_reports WHERE md5 = mymd5 LOOP
         classification := lower(myrow.classification);
         IF classification LIKE '%allaple%' THEN
             allaple := allaple + 1; 
         END IF;
     END LOOP;
     IF allaple > ALLAPLE_THRESHOLD THEN
         RAISE NOTICE 'Sample % is ALLAPLE, updating.', mymd5;
         SELECT set_sample(mymd5, WORM_TYPE, ALLAPLE_ID) INTO val;
         UPDATE av_reports SET processed = true WHERE md5 = mymd5;
     END IF;
     RETURN val;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: process_available_reports(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION process_available_reports() RETURNS integer
    AS $$
DECLARE
    myrow RECORD;
    val boolean DEFAULT 'false';
    counter integer DEFAULT 0;
BEGIN
    FOR myrow IN 
        SELECT id FROM reports 
        WHERE available = 'true' AND processed = 'false' AND (
              provider = 'CWSandbox' or provider = 'VirusTotal') 
        order by id desc limit 100
    LOOP
        RAISE NOTICE 'Processing report %', myrow.id;
        SELECT process_report(myrow.id) INTO val;
        IF val THEN
            counter := counter + 1;
        END IF;
    END LOOP;
    FOR myrow IN
        SELECT DISTINCT md5 FROM av_reports WHERE processed = 'false'
    LOOP
        RAISE NOTICE 'Processing av reports for %', myrow.md5;
        SELECT process_av_report(myrow.md5) INTO val;
        IF val THEN
            counter := counter + 1;
        END IF;
    END LOOP; 
    RETURN counter;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: process_report(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION process_report() RETURNS trigger
    AS $$
BEGIN
PERFORM process_report(NEW.id);
RETURN NULL; -- ignored, this is run as AFTER trigger
END;
$$
    LANGUAGE plpgsql;

--
-- Name: process_report(integer); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION process_report(id integer) RETURNS boolean
    AS $_$
import xml.dom.minidom
from xml.dom.minidom import Node
import urllib
import os
import re
import grp

BASE = '/var/www/uploads/'
GROUP = 'www-data'
ALLAPLE_MUTEX_NAME = 'jhdheddfffffhjk5trh'
ALLAPLE_FILE_NAME = 'urdvxc.exe'
ALLAPLE_ID = 1
ALLAPLE_THRESHOLD = 3

def prepare_query(name, query, args):
    try:
        if not SD.has_key(name):
            SD[name] = plpy.prepare(query, args)
            return
    except NameError:
        return

def execute_query(name, args):
    try:
        if SD.has_key(name):
            return plpy.execute(SD[name], args)
        else:
            plpy.fatal("Query '%s' does not exist in SD, aborting." %(name))
    except NameError:
         print "Running '%s' with args: %s" %(name, args)
         return None

def execute_function(name, args):
    res = execute_query(name, args)
    if res != None:
        return res[0][name]
    else:
        return None

def getAttribute(attributes, key):
    if attributes.has_key(key):
        return attributes[key].value
    else:
        return None

def getText(node):
    val = ''
    for node3 in node.childNodes:
        if node3.nodeType == Node.TEXT_NODE:
            val += node3.data
    return val

"""
def commit_report(md5, provider, type, id):
    prepare_query('set_sample', 'UPDATE samples SET malware_id = $2, type = $3 WHERE md5 = $1', ['text','int4','text'])
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider = $2", [ 'text', 'malware_type' ]) 
    execute_query('set_sample', [md5, id, type])
    execute_query('set_processed', [md5, provider])
"""

def process_virustotal(md5, report_file):
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider::text = $2", [ 'text', 'text' ]) 
    prepare_query('add_av_report', 'INSERT INTO av_reports (md5, scanner, scanner_version, signature_version, classification) VALUES ($1, $2, $3, $4, $5)', ['text', 'text', 'text', 'date', 'text'])
    allaple_count = 0
    root_n = xml.dom.minidom.parse(report_file)
    analysis_n = root_n.getElementsByTagName('analysis')[0]
    virusscan_n = analysis_n.getElementsByTagName('virusscan_section')[0]
    for scanner_n in virusscan_n.getElementsByTagName('scanner'):
        scanner = getAttribute(scanner_n.attributes, 'name')
        scanner_version = getAttribute(scanner_n.attributes, 'application_version')
        signature_version = getAttribute(scanner_n.attributes, 'signature_file_version')
        classification = getText(scanner_n.getElementsByTagName('classification')[0])
        execute_query('add_av_report', [ md5, scanner, scanner_version, signature_version, classification ])
    execute_query('set_processed', [ md5, 'VirusTotal' ])
    return 'true'

def process_cwsandbox(md5, report_file):
    prepare_query('add_cnc', 'SELECT add_cnc($1, $2, $3, $4)', [ 'inet', 'int4', 'cnc_type', 'varchar' ])
    prepare_query('add_http_bot', 'SELECT add_http_bot($1, $2)', [ 'int4', 'timestamp' ])
    prepare_query('add_http_cmd', 'SELECT add_http_cmd($1, $2, $3, $4, $5)', [ 'int4', 'varchar', 'varchar', 'varchar', 'text' ])
    prepare_query('add_irc_bot', 'SELECT add_irc_bot($1, $2, $3, $4, $5, $6, $7, $8, $9)', [ 'int4', 'varchar',  'varchar', 'varchar', 'varchar', 'varchar', 'varchar', 'bool', 'timestamp' ])
    prepare_query('add_irc_channel', 'SELECT add_irc_channel($1, $2, $3, $4)', [ 'int4', 'varchar', 'varchar', 'text' ])
    prepare_query('add_irc_privmsg', 'SELECT add_irc_privmsg($1, $2, $3)', [ 'int4', 'varchar', 'varchar' ])
    prepare_query('add_target','SELECT add_target($1, $2, $3, $4, $5, $6, $7)', [ 'inet', 'int4', 'varchar', 'int4', 'timestamp', 'varchar', 'bool' ])
    prepare_query('set_sample', 'SELECT set_sample($1, $2, $3)', [ 'text','malware_type','int4'])
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider::text = $2", [ 'text', 'text' ]) 

    dnsqueries = []
    revdnsqueries = []
    allaple_mutex = False
    allaple_file = False
    if not SD.has_key('winfile_r'):
        SD['winfile_r'] = re.compile('^.*\\\\(.*)$')
    root_n = xml.dom.minidom.parse(report_file)
    analysis_n = root_n.getElementsByTagName("analysis")[0]
    analysis_date = getAttribute(analysis_n.attributes, 'time')
    if not SD.has_key('date_r'):
        SD['date_r'] = re.compile('^([0-9]+)/([0-9]+)/([0-9]+)\ (.*)$')
    date_m = SD['date_r'].search(analysis_date)
    if date_m != None:
        month = date_m.group(1)
        day = date_m.group(2)
        year = date_m.group(3)
        time = date_m.group(4)
        analysis_date = "%s.%s.%s %s" %(day, month, year, time)
        plpy.notice("braindead date format, workaround activated; fixed date is '%s'" %(analysis_date))
    processes_n = analysis_n.getElementsByTagName("processes")[0]
    targets = []
    malwares = []
    for process_n in processes_n.getElementsByTagName("process"):
        ## FILESYSTEM ##
        filesys_n = process_n.getElementsByTagName("filesystem_section")
        if len(filesys_n) > 0:
            copy_n = filesys_n[0].getElementsByTagName("copy_file")
            for copy_op in copy_n:
                dest = copy_op.attributes['dstfile'].value
                m = SD['winfile_r'].search(dest)
                if m != None:
                    name = m.group(1)
                    if name == ALLAPLE_FILE_NAME:
                        allaple_file = True
        ## MUTEX ##
        mutex_n = process_n.getElementsByTagName("mutex_section")
        if len(mutex_n) > 0:
            create_mutex_n = mutex_n[0].getElementsByTagName("create_mutex")
            for mutex in create_mutex_n:
                name = getAttribute(mutex.attributes,'name')
                if name == ALLAPLE_MUTEX_NAME:
                    allaple_mutex = True 
        if allaple_file and allaple_mutex:
            execute_query('set_sample', [ md5, 'worm', ALLAPLE_ID ])
            execute_query('set_processed', [ md5, 'CWSandbox' ])
            return 'true'
        ## WINSOCK ##
        winsock_n = process_n.getElementsByTagName("winsock_section")
        if len(winsock_n) > 0:
            conns_unk_n = winsock_n[0].getElementsByTagName("connections_unknown")
            if len(conns_unk_n) > 0:
                conn_unk_n = conns_unk_n[0]
                for conn_n in conn_unk_n.getElementsByTagName("connection"):
                    for gethostbyname_n in conn_n.getElementsByTagName("gethostbyname"):
                        gethostbyname_a = gethostbyname_n.attributes
                        host = getAttribute(gethostbyname_a, 'requested_host')
                        address = [ getAttribute(gethostbyname_a, 'resulting_addr') ]
                        more_n = gethostbyname_n.getElementsByTagName("more_resulting_addr")
                        if len(more_n) > 0:
                            for node in more_n[0].getElementsByTagName("resulting_addr"):
                                t = getText(node)
                                if t != '':
                                    address.append(t)
                        dnsqueries.append([host, address])
                    for gethostbyaddr_n in conn_n.getElementsByTagName("gethostbyaddr"):
                        gethostbyaddr_a = gethostbyaddr_n.attributes
                        address = getAttribute(gethostbyaddr_a, 'requested_addr')
                        host = getAttribute(gethostbyaddr_a, 'resulting_host')
                        revdnsqueries.append([address, host])
                
            conns_out_n = winsock_n[0].getElementsByTagName("connections_outgoing")
            if len(conns_out_n) > 0:
                conn_out_n = conns_out_n[0]
                for conn_n in conn_out_n.getElementsByTagName("connection"):
                    transport = conn_n.attributes['transportprotocol'].value
                    remote_addr = conn_n.attributes['remoteaddr'].value
                    remote_port = int(conn_n.attributes['remoteport'].value)
                    remote_dns = ''
                    if conn_n.attributes['connectionestablished'].value == '0':
                        established = False
                    else:
                        established = True
                    for query in dnsqueries:
                        host = query[0]
                        for address in query[1]:
                            if remote_addr == address:
                                remote_dns = host
                                break
                        if remote_dns != '':
                            break
                    if conn_n.attributes.has_key('protocol'):
                         protocol = conn_n.attributes['protocol'].value
                         if protocol == 'IRC':
                             cnc = [ remote_addr, remote_port, 'irc', remote_dns ]
                             
                             ircdata_n = conn_n.getElementsByTagName("irc_data")[0]
                             ircdata_a = ircdata_n.attributes
                             hostname = getAttribute(ircdata_a, 'hostname')
                             servername = getAttribute(ircdata_a, 'servername')
                             username = getAttribute(ircdata_a, 'username')
                             realname = getAttribute(ircdata_a, 'realname')
                             #FIXME workaround for CWsandbox parsing bug
                             if realname == '-Service':
                                 realname = '-Service Pack 2'
                             password = getAttribute(ircdata_a, 'password')
                             nickname = getAttribute(ircdata_a, 'nick')
                             if getAttribute(ircdata_a, 'non_rfc_conform') == '0':
                                 rfc_compliant = 'true'
                             else:
                                 rfc_compliant = 'false'
                             bot_data = [hostname, realname, nickname, username, password, servername, rfc_compliant, analysis_date]
                             channels = []
                             for channel_n in ircdata_n.getElementsByTagName("channel"):
                                 channel_a = channel_n.attributes
                                 chan_name = getAttribute(channel_a, 'name')
                                 chan_password = getAttribute(channel_a, 'password')
                                 chan_topic = getAttribute(channel_a, 'topic_deleted')
                                 channels.append([chan_name, chan_password, chan_topic])
                             notices = []
                             for notice_n in ircdata_n.getElementsByTagName("notice_deleted"):
                                 notice_value = getAttribute(notice_n.attributes, 'value')
                                 notices.append(['n', notice_value])
                             for priv_n in ircdata_n.getElementsByTagName("privmsg_deleted"):
                                 priv_value = getAttribute(priv_n.attributes, 'value')
                                 notices.append(['p', priv_value])
                             malwares.append([ 'irc', cnc, bot_data, channels, notices ])
                         elif protocol == 'HTTP':
                             cnc = [ remote_addr, remote_port, 'http', remote_dns ]
                             bot_data = [ analysis_date ]
                             httpcmds = []
                             httpdata_n = conn_n.getElementsByTagName("http_data")[0]
                             for httpcmd_n in httpdata_n.getElementsByTagName('http_cmd'):
                                 httpcmd_a = httpcmd_n.attributes
                                 url = getAttribute(httpcmd_a, 'url')
                                 method = getAttribute(httpcmd_a, 'method')
                                 http_version = getAttribute(httpcmd_a, 'http_version')
                                 headers = []
                                 headers_n = httpcmd_n.getElementsByTagName('header_data')
                                 if len(headers_n) > 0:
                                     for header_n in headers_n[0].getElementsByTagName('header'):
                                         t = getText(header_n)
                                         if t != '':
                                             headers.append(t)
                                 headers_s = '\n'.join(headers)
                                 httpcmds.append([url, method, http_version, headers_s])
                             malwares.append(['http', cnc, bot_data, httpcmds ])
                         else:
                             targets.append([ remote_addr, remote_port, transport, analysis_date, remote_dns, established ])
                    else:
                        targets.append([ remote_addr, remote_port, transport, analysis_date, remote_dns, established ])
        ## ICMP ##
        icmp_n = process_n.getElementsByTagName("icmp_section")
        if len(icmp_n) > 0:
            for ping_n in icmp_n[0].getElementsByTagName("ping"):
                ping_a = ping_n.attributes
                host = getAttribute(ping_a, 'host')
                remote_dns = ''
                for query in dnsqueries:
                    host = query[0]
                    for address in query[1]:
                        if remote_addr == address:
                            remote_dns = host
                            break
                    if remote_dns != '':
                        break
                targets.append([ host, None, 'ICMP', analysis_date, remote_dns, None ])
    bot_id = None
    for malware in malwares:
        cnc_id = execute_function('add_cnc', malware[1])
        type = malware[0]
        bot_data = malware[2]
        bot_data.insert(0, cnc_id)
        if type == 'irc':
            bot_id = execute_function('add_irc_bot', bot_data)
            if len(malware[3]) > 0:
                channels = malware[3][0]
                channels.insert(0, bot_id)
                execute_query('add_irc_channel', channels)
            if len(malware[4]) > 0:
                notices = malware[4][0]
                notices.insert(0, bot_id)
                execute_query('add_irc_privmsg', notices)
            execute_query('set_sample', [ md5, 'irc_bot', bot_id ])
        elif type == 'http':
            bot_id = execute_function('add_http_bot', bot_data)
            if len(malware[3]) > 0:
                httpcmds = malware[3][0]          
                httpcmds.insert(0, bot_id)
                execute_query('add_http_cmd', httpcmds)
            execute_query('set_sample', [ md5, 'http_bot', bot_id ])
        else:
            plpy.warning("unknown malware '%s'" %(type))
    if bot_id != None:
        for target in targets:
            target.insert(3, bot_id)
            execute_query('add_target', target)
        execute_query('set_processed', [ md5, 'CWSandbox' ])
    return 'true'

prepare_query('select', 'SELECT md5, provider, task_id, analysis_id, password, processed FROM reports WHERE id = $1', ['int4'])
res = execute_query('select', [ id ])
if res[0]['processed']:
    return 'false'
md5 = res[0]['md5']
provider = res[0]['provider']
file_gid = grp.getgrnam(GROUP)[2]
if provider == 'CWSandbox':
    analysis_id = res[0]['analysis_id']
    password = res[0]['password']
    xml_url = "http://www.cwsandbox.org/?page=analysis&id=%s&format=xml&password=%s" %(analysis_id, password)
    xml_file = BASE + md5 + '.xml'
    pcap_url = "http://www.cwsandbox.org/?page=download&dltype=pcap&id=%s&password=%s" %(analysis_id,password)
    pcap_file = BASE + md5 + '.pcap'
    if not os.path.exists(xml_file):
        urllib.urlretrieve(xml_url, xml_file)
        os.chmod(xml_file, 0660)
        os.chown(xml_file, -1, file_gid)
    if not os.path.exists(pcap_file):
        urllib.urlretrieve(pcap_url, pcap_file)
        os.chmod(pcap_file, 0660)
        os.chown(pcap_file, -1, file_gid)
    val = process_cwsandbox(md5, xml_file)
    return val
elif provider == 'VirusTotal':
    analysis_id = res[0]['analysis_id']
    password = res[0]['password']
    xml_url = "http://www.cwsandbox.org/?page=analysis&id=%s&format=xml&password=%s" %(analysis_id, password)
    xml_file = "%svirustotal/%s.xml" %(BASE,md5)
    if not os.path.exists(xml_file):
        urllib.urlretrieve(xml_url, xml_file)
        os.chmod(xml_file, 0660)
        os.chown(xml_file, -1, file_gid)
    val = process_virustotal(md5, xml_file)        
    return val
$_$
    LANGUAGE plpythonu;

--
-- Name: process_report2(integer); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION process_report2(id integer) RETURNS boolean
    AS $_$
import xml.dom.minidom
from xml.dom.minidom import Node
import urllib
import os
import re
import grp

BASE = '/var/www/uploads/'
GROUP = 'www-data'
ALLAPLE_MUTEX_NAME = 'jhdheddfffffhjk5trh'
ALLAPLE_FILE_NAME = 'urdvxc.exe'
ALLAPLE_ID = 1
ALLAPLE_THRESHOLD = 3

def prepare_query(name, query, args):
    try:
        if not SD.has_key(name):
            SD[name] = plpy.prepare(query, args)
            return
    except NameError:
        return

def execute_query(name, args):
    try:
        if SD.has_key(name):
            return plpy.execute(SD[name], args)
        else:
            plpy.fatal("Query '%s' does not exist in SD, aborting." %(name))
    except NameError:
         print "Running '%s' with args: %s" %(name, args)
         return None

def execute_function(name, args):
    res = execute_query(name, args)
    if res != None:
        return res[0][name]
    else:
        return None

def getAttribute(attributes, key):
    if attributes.has_key(key):
        return attributes[key].value
    else:
        return None

def getText(node):
    val = ''
    for node3 in node.childNodes:
        if node3.nodeType == Node.TEXT_NODE:
            val += node3.data
    return val

def commit_report(md5, provider, type, id):
    prepare_query('set_sample', 'UPDATE samples SET malware_id = $2, type = $3 WHERE md5 = $1', ['text','int4','text'])
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider = $2", [ 'text', 'text' ]) 
    execute_query('set_sample', [md5, id, type])
    execute_query('set_processed', [md5, provider])

def process_virustotal(md5, report_file):
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider = $2", [ 'text', 'text' ]) 
    prepare_query('add_av_report', 'INSERT INTO av_reports (md5, scanner, scanner_version, signature_version, classification) VALUES ($1, $2, $3, $4, $5)', ['text', 'text', 'text', 'date', 'text'])
    allaple_count = 0
    root_n = xml.dom.minidom.parse(report_file)
    analysis_n = root_n.getElementsByTagName('analysis')[0]
    virusscan_n = analysis_n.getElementsByTagName('virusscan_section')[0]
    for scanner_n in virusscan_n.getElementsByTagName('scanner'):
        scanner = getAttribute(scanner_n.attributes, 'name')
        scanner_version = getAttribute(scanner_n.attributes, 'application_version')
        signature_version = getAttribute(scanner_n.attributes, 'signature_file_version')
        classification = getText(scanner_n.getElementsByTagName('classification')[0])
        execute_query('add_av_report', [ md5, scanner, scanner_version, signature_version, classification ])
    execute_query('set_processed', [ md5, 'VirusTotal' ])
    return 'true'

def process_cwsandbox(md5, report_file):
    prepare_query('add_cnc', 'SELECT add_cnc($1, $2, $3, $4)', [ 'inet', 'int4', 'varchar', 'varchar' ])
    prepare_query('add_http_bot', 'SELECT add_http_bot($1, $2)', [ 'int4', 'timestamp' ])
    prepare_query('add_http_cmd', 'SELECT add_http_cmd($1, $2, $3, $4, $5)', [ 'int4', 'varchar', 'varchar', 'varchar', 'text' ])
    prepare_query('add_irc_bot', 'SELECT add_irc_bot($1, $2, $3, $4, $5, $6, $7, $8, $9)', [ 'int4', 'varchar',  'varchar', 'varchar', 'varchar', 'varchar', 'varchar', 'bool', 'timestamp' ])
    prepare_query('add_irc_channel', 'SELECT add_irc_channel($1, $2, $3, $4)', [ 'int4', 'varchar', 'varchar', 'text' ])
    prepare_query('add_irc_privmsg', 'SELECT add_irc_privmsg($1, $2, $3)', [ 'int4', 'varchar', 'varchar' ])
    prepare_query('add_target','SELECT add_target($1, $2, $3, $4, $5, $6, $7)', [ 'inet', 'int4', 'varchar', 'int4', 'timestamp', 'varchar', 'bool' ])
    prepare_query('set_sample', 'SELECT set_sample($1, $2, $3)', [ 'text','text','int4'])
    prepare_query('set_processed', "UPDATE reports SET processed = 'true' WHERE md5 = $1 and provider = $2", [ 'text', 'text' ]) 

    dnsqueries = []
    revdnsqueries = []
    allaple_mutex = False
    allaple_file = False
    if not SD.has_key('winfile_r'):
        SD['winfile_r'] = re.compile('^.*\\\\(.*)$')
    root_n = xml.dom.minidom.parse(report_file)
    analysis_n = root_n.getElementsByTagName("analysis")[0]
    analysis_date = getAttribute(analysis_n.attributes, 'time')
    if not SD.has_key('date_r'):
        SD['date_r'] = re.compile('^([0-9]+)/([0-9]+)/([0-9]+)\ (.*)$')
    date_m = SD['date_r'].search(analysis_date)
    if date_m != None:
        month = date_m.group(1)
        day = date_m.group(2)
        year = date_m.group(3)
        time = date_m.group(4)
        analysis_date = "%s.%s.%s %s" %(day, month, year, time)
        plpy.notice("braindead date format, workaround activated; fixed date is '%s'" %(analysis_date))
    processes_n = analysis_n.getElementsByTagName("processes")[0]
    targets = []
    malwares = []
    for process_n in processes_n.getElementsByTagName("process"):
        ## FILESYSTEM ##
        filesys_n = process_n.getElementsByTagName("filesystem_section")
        if len(filesys_n) > 0:
            copy_n = filesys_n[0].getElementsByTagName("copy_file")
            for copy_op in copy_n:
                dest = copy_op.attributes['dstfile'].value
                m = SD['winfile_r'].search(dest)
                if m != None:
                    name = m.group(1)
                    if name == ALLAPLE_FILE_NAME:
                        allaple_file = True
        ## MUTEX ##
        mutex_n = process_n.getElementsByTagName("mutex_section")
        if len(mutex_n) > 0:
            create_mutex_n = mutex_n[0].getElementsByTagName("create_mutex")
            for mutex in create_mutex_n:
                name = getAttribute(mutex.attributes,'name')
                if name == ALLAPLE_MUTEX_NAME:
                    allaple_mutex = True 
        if allaple_file and allaple_mutex:
            execute_query('set_sample', [ md5, 'worm', ALLAPLE_ID ])
            execute_query('set_processed', [ md5, 'CWSandbox' ])
            return 'true'
        ## WINSOCK ##
        winsock_n = process_n.getElementsByTagName("winsock_section")
        if len(winsock_n) > 0:
            conns_unk_n = winsock_n[0].getElementsByTagName("connections_unknown")
            if len(conns_unk_n) > 0:
                conn_unk_n = conns_unk_n[0]
                for conn_n in conn_unk_n.getElementsByTagName("connection"):
                    for gethostbyname_n in conn_n.getElementsByTagName("gethostbyname"):
                        gethostbyname_a = gethostbyname_n.attributes
                        host = getAttribute(gethostbyname_a, 'requested_host')
                        address = [ getAttribute(gethostbyname_a, 'resulting_addr') ]
                        more_n = gethostbyname_n.getElementsByTagName("more_resulting_addr")
                        if len(more_n) > 0:
                            for node in more_n[0].getElementsByTagName("resulting_addr"):
                                t = getText(node)
                                if t != '':
                                    address.append(t)
                        dnsqueries.append([host, address])
                    for gethostbyaddr_n in conn_n.getElementsByTagName("gethostbyaddr"):
                        gethostbyaddr_a = gethostbyaddr_n.attributes
                        address = getAttribute(gethostbyaddr_a, 'requested_addr')
                        host = getAttribute(gethostbyaddr_a, 'resulting_host')
                        revdnsqueries.append([address, host])
                
            conns_out_n = winsock_n[0].getElementsByTagName("connections_outgoing")
            if len(conns_out_n) > 0:
                conn_out_n = conns_out_n[0]
                for conn_n in conn_out_n.getElementsByTagName("connection"):
                    transport = conn_n.attributes['transportprotocol'].value
                    remote_addr = conn_n.attributes['remoteaddr'].value
                    remote_port = int(conn_n.attributes['remoteport'].value)
                    remote_dns = ''
                    if conn_n.attributes['connectionestablished'].value == '0':
                        established = False
                    else:
                        established = True
                    for query in dnsqueries:
                        host = query[0]
                        for address in query[1]:
                            if remote_addr == address:
                                remote_dns = host
                                break
                        if remote_dns != '':
                            break
                    if conn_n.attributes.has_key('protocol'):
                         protocol = conn_n.attributes['protocol'].value
                         if protocol == 'IRC':
                             cnc = [ remote_addr, remote_port, 'irc', remote_dns ]
                             
                             ircdata_n = conn_n.getElementsByTagName("irc_data")[0]
                             ircdata_a = ircdata_n.attributes
                             
                             hostname = getAttribute(ircdata_a, 'hostname')
                             servername = getAttribute(ircdata_a, 'servername')
                             username = getAttribute(ircdata_a, 'username')
                             realname = getAttribute(ircdata_a, 'realname')
                             #FIXME workaround for CWsandbox parsing bug
                             if realname == '-Service':
                                 realname = '-Service Pack 2'
                             password = getAttribute(ircdata_a, 'password')
                             nickname = getAttribute(ircdata_a, 'nick')
                             if getAttribute(ircdata_a, 'non_rfc_conform') == '0':
                                 rfc_compliant = 'true'
                             else:
                                 rfc_compliant = 'false'
                             bot_data = [hostname, realname, nickname, username, password, servername, rfc_compliant, analysis_date]
                             channels = []
                             for channel_n in ircdata_n.getElementsByTagName("channel"):
                                 channel_a = channel_n.attributes
                                 chan_name = getAttribute(channel_a, 'name')
                                 chan_password = getAttribute(channel_a, 'password')
                                 chan_topic = getAttribute(channel_a, 'topic_deleted')
                                 channels.append([chan_name, chan_password, chan_topic])
                             notices = []
                             for notice_n in ircdata_n.getElementsByTagName("notice_deleted"):
                                 notice_value = getAttribute(notice_n.attributes, 'value')
                                 notices.append(['n', notice_value])
                             for priv_n in ircdata_n.getElementsByTagName("privmsg_deleted"):
                                 priv_value = getAttribute(priv_n.attributes, 'value')
                                 notices.append(['p', priv_value])
                             malwares.append([ 'irc', cnc, bot_data, channels, notices ])
                         elif protocol == 'HTTP':
                             cnc = [ remote_addr, remote_port, 'http', remote_dns ]
                             bot_data = [ analysis_date ]
                             httpcmds = []
                             httpdata_n = conn_n.getElementsByTagName("http_data")[0]
                             for httpcmd_n in httpdata_n.getElementsByTagName('http_cmd'):
                                 httpcmd_a = httpcmd_n.attributes
                                 url = getAttribute(httpcmd_a, 'url')
                                 method = getAttribute(httpcmd_a, 'method')
                                 http_version = getAttribute(httpcmd_a, 'http_version')
                                 headers = []
                                 headers_n = httpcmd_n.getElementsByTagName('header_data')
                                 if len(headers_n) > 0:
                                     for header_n in headers_n[0].getElementsByTagName('header'):
                                         t = getText(header_n)
                                         if t != '':
                                             headers.append(t)
                                 headers_s = '\n'.join(headers)
                                 httpcmds.append([url, method, http_version, headers_s])
                             malwares.append(['http', cnc, bot_data, httpcmds ])
                         else:
                             targets.append([ remote_addr, remote_port, transport, analysis_date, remote_dns, established ])
                    else:
                        targets.append([ remote_addr, remote_port, transport, analysis_date, remote_dns, established ])
        ## ICMP ##
        icmp_n = process_n.getElementsByTagName("icmp_section")
        if len(icmp_n) > 0:
            for ping_n in icmp_n[0].getElementsByTagName("ping"):
                ping_a = ping_n.attributes
                host = getAttribute(ping_a, 'host')
                remote_dns = ''
                for query in dnsqueries:
                    host = query[0]
                    for address in query[1]:
                        if remote_addr == address:
                            remote_dns = host
                            break
                    if remote_dns != '':
                        break
                targets.append([ host, None, 'ICMP', analysis_date, remote_dns, None ])
    bot_id = None
    for malware in malwares:
        cnc_id = execute_function('add_cnc', malware[1])
        type = malware[0]
        bot_data = malware[2]
        bot_data.insert(0, cnc_id)
        if type == 'irc':
            bot_id = execute_function('add_irc_bot', bot_data)
            if len(malware[3]) > 0:
                channels = malware[3][0]
                channels.insert(0, bot_id)
                execute_query('add_irc_channel', channels)
            if len(malware[4]) > 0:
                notices = malware[4][0]
                notices.insert(0, bot_id)
                execute_query('add_irc_privmsg', notices)
            execute_query('set_sample', [ md5, 'irc_bot', bot_id ])
        elif type == 'http':
            bot_id = execute_function('add_http_bot', bot_data)
            if len(malware[3]) > 0:
                httpcmds = malware[3][0]          
                httpcmds.insert(0, bot_id)
                execute_query('add_http_cmd', httpcmds)
            execute_query('set_sample', [ md5, 'http_bot', bot_id ])
        else:
            plpy.warning("unknown malware '%s'" %(type))
    if bot_id != None:
        for target in targets:
            target.insert(3, bot_id)
            execute_query('add_target', target)
        execute_query('set_processed', [ md5, 'CWSandbox' ])
    return 'true'

prepare_query('select', 'SELECT md5, provider, task_id, analysis_id, password, processed, report FROM reports WHERE id = $1', ['int4'])
res = execute_query('select', [ id ])
if res[0]['processed']:
    return 'false'
md5 = res[0]['md5']
provider = res[0]['provider']
file_gid = grp.getgrnam(GROUP)[2]
if provider == 'CWSandbox':
    analysis_id = res[0]['analysis_id']
    password = res[0]['password']
    report = res[0]['report']
    if report = None:
        prepare_query("UPDATE reports SET report = XMLPARSE(DOCUMENT $3) WHERE md5 = $1 AND provider = $2", [ 'text', 'text', 'text' ])
        xml_url = "http://www.cwsandbox.org/?page=analysis&id=%s&format=xml&password=%s" %(analysis_id, password)
        f = urllib.urlopen(xml_url)
        report = f.read()
        f.close()
        execute_query('add_xml', [ md5, provider, report ])
    
    xml_file = BASE + md5 + '.xml'
    pcap_url = "http://www.cwsandbox.org/?page=download&dltype=pcap&id=%s&password=%s" %(analysis_id,password)
    pcap_file = BASE + md5 + '.pcap'
    if not os.path.exists(xml_file):
        urllib.urlretrieve(xml_url, xml_file)
        os.chmod(xml_file, 0660)
        os.chown(xml_file, -1, file_gid)
    if not os.path.exists(pcap_file):
        urllib.urlretrieve(pcap_url, pcap_file)
        os.chmod(pcap_file, 0660)
        os.chown(pcap_file, -1, file_gid)
    val = process_cwsandbox(md5, xml_file)
    return val
elif provider == 'VirusTotal':
    analysis_id = res[0]['analysis_id']
    password = res[0]['password']
    xml_url = "http://www.cwsandbox.org/?page=analysis&id=%s&format=xml&password=%s" %(analysis_id, password)
    xml_file = "%svirustotal/%s.xml" %(BASE,md5)
    if not os.path.exists(xml_file):
        urllib.urlretrieve(xml_url, xml_file)
        os.chmod(xml_file, 0660)
        os.chown(xml_file, -1, file_gid)
    val = process_virustotal(md5, xml_file)        
    return val
$_$
    LANGUAGE plpythonu;

--
-- Name: reset_processed(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION reset_processed() RETURNS boolean
    AS $$
truncate table cncs, http_cmds, irc_channels, irc_privmsgs, targets, samples_ref cascade;
update reports set processed = false where processed = true;
select setval('cncs_id_seq', 1, false);
select setval('http_cmds_id_seq', 1, false);
select setval('irc_channels_id_seq', 1, false);
select setval('malware_id_seq', 1, true);
select setval('targets_id_seq', 1, false);
select true;
$$
    LANGUAGE sql;

--
-- Name: set_geo(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION set_geo() RETURNS trigger
    AS $$
import GeoIP
if TD['event'] == 'INSERT':
    if not SD.has_key('geoip'):
        SD['geoip'] = GeoIP.open('/home/malware/GeoLiteCity.dat', GeoIP.GEOIP_MEMORY_CACHE)
    data = SD['geoip'].record_by_addr(TD['new']['address'])
    if data != None:
        TD['new']['country'] = data['country_code']
        TD['new']['latitude'] = data['latitude']
        TD['new']['longitude'] = data['longitude']
        return 'MODIFY'
return 'OK'
$$
    LANGUAGE plpythonu;

--
-- Name: set_sample(character, malware_type, integer); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION set_sample(mysample character, mytype malware_type, mymalware_id integer) RETURNS boolean
    AS $$
DECLARE
       temprow RECORD;
BEGIN
     SELECT * INTO temprow FROM samples_ref WHERE sample = mysample AND malware_id = mymalware_id;
     IF NOT FOUND THEN
         INSERT INTO samples_ref (sample, type, malware_id) VALUES (mysample, mytype, mymalware_id);
         RETURN TRUE;
     ELSE
         RAISE WARNING 'Malware % (type %) is already linked to sample %, skipping', mymalware_id, mytype, mysample;
         RETURN FALSE;
     END IF;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: submit_to_anubis(character); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION submit_to_anubis(md5 character) RETURNS boolean
    AS $_$
import sys
sys.path.append('/home/malware')
import httputils
from pg import unescape_bytea

if not SD.has_key('selcheck'):
    SD['selcheck'] = plpy.prepare(
        "SELECT md5 FROM reports WHERE md5 = $1 and provider = 'Anubis'",
        [ 'text' ])

if len(plpy.execute(SD['selcheck'], [ md5 ])) > 0:
    return 'false'

if not SD.has_key('submit_url'):
    res = plpy.execute("select value as submit_url from settings where key = 'ANUBIS_SUBMIT_URL'")
    SD['submit_url'] = res[0]['submit_url']

if not SD.has_key('sample_check'):
    SD['sample_check'] = plpy.prepare('SELECT sample FROM samples WHERE md5 = $1 AND sample IS NOT NULL', [ "text" ])

res = plpy.execute(SD['sample_check'], [ md5 ])
if len(res) > 0:
    file = md5
    content = unescape_bytea(res[0]['sample'])
else:
    file = "/var/www/uploads/%s" %(md5)
    fd = open(file, "rb")
    content = fd.read()
    fd.close()

task_id = 0
post_data = {}
post_data["notification"] = "email"
post_data["email"] = "malware@example.com"
post_data["test_subject"] = { "content" : content, "filename" : file }
response = httputils.httprequest(SD['submit_url'], post_data)
if (response.status == 302 and 
    response.getheader("taskid", "DEFAULT") != "DEFAULT"):
    task_id = response.getheader('taskid')
    if not SD.has_key('insert'):
        SD['insert'] = plpy.prepare("INSERT INTO reports (md5, provider, task_id, available) VALUES ($1, 'Anubis', $2, false)", [ "text", "text" ])
    plpy.execute(SD['insert'], [ md5, task_id ])
    return 'true'
else:
    plpy.notice(response.status)
    plpy.notice(response.read())
    return 'false'
$_$
    LANGUAGE plpythonu;

--
-- Name: submit_to_clamav(character); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION submit_to_clamav(md5 character) RETURNS boolean
    AS $_$
import pyclamd
from pyclamd import ScanError
import os
from time import strftime, strptime

SAMPLES_PATH = '/var/www/uploads'
ALLAPLE_ID = 1
PROXIM_ID = 10

def prepare_query(name, query, args):
    try:
        if not SD.has_key(name):
            SD[name] = plpy.prepare(query, args)
            return
    except NameError:
        return

def execute_query(name, args):
    try:
        if SD.has_key(name):
            return plpy.execute(SD[name], args)
        else:
            plpy.fatal("Query '%s' does not exist in SD, aborting." %(name))
    except NameError:
         print "Running '%s' with args: %s" %(name, args)
         return None

try:
    pyclamd.ping()
except ScanError:
    pyclamd.init_network_socket('localhost', 3310)

version = pyclamd.version().split('/')
scanner_version = version[0].split(' ')[1]
signature_version = strftime('%Y%m%d',strptime(version[2]))

prepare_query('getsample', 'SELECT sample FROM samples WHERE md5 = $1', [ 'text' ])
#prepare_query('checkvariant', 'SELECT * FROM clamav_variants WHERE name = $1 AND variant = $2', [ 'text', 'text' ])
prepare_query('newvariant', 'INSERT INTO clamav_variants(name, variant) VALUES ($1, $2)', [ 'text', 'text' ])
prepare_query('setsample', 'SELECT set_sample($1, $2, $3)', [ 'text','malware_type','int4'])
prepare_query('addreport', 'INSERT INTO av_reports (md5, scanner, scanner_version, signature_version, classification) VALUES ($1, $2, $3, $4, $5)', [ 'text', 'text', 'text', 'date', 'text'])

res = execute_query('getsample', [ md5 ])
if len(res) == 0:
    plpy.error('Unknown sample %s' %(md5))
else:
    sample = res[0]['sample']         
    file = "%s/%s" %(SAMPLES_PATH, md5)
    try:
        if sample != None:
           result = pyclamd.scan_stream(sample)
        elif os.path.exists(file):
           result = pyclamd.scan_file(file)
        else:
           plpy.error('Cannot find sample file %s' %(md5))
    except ScanError:
        plpy.warning('Error scanning sample %s' %(md5))
        return True
        
    if result == None:
        execute_query('addreport', [ md5, 'ClamAV-local', scanner_version, signature_version, 'OK' ])
        return True
    else:
        virus = result.values()[0].strip() 
        execute_query('addreport', [ md5, 'ClamAV-local', scanner_version, signature_version, virus ])
        lovirus = virus.lower()
        if lovirus.find('allaple') != -1:
            # process allaple
            execute_query('setsample', [ md5, 'worm', ALLAPLE_ID ])
            return False
        elif virus == 'W32.Virut.A':
            #res = execute_query('checkvariant', [ 'virut', virus ])
            execute_query('setsample', [ md5, 'irc_bot', PROXIM_ID ])
            return False
        else:
            return True
            """
            if len(res) == 0:
                # new variant, register and process as usual
                execute_query('newvariant', [ 'virut', virus ])
                return True    
            else:
                # old variant, skip
                execute_query('setsample', [ md5, 'irc_bot', ALLAPLE_ID ])
                return False
            """
$_$
    LANGUAGE plpythonu;

--
-- Name: submit_to_cwsandbox(character); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION submit_to_cwsandbox(md5 character) RETURNS boolean
    AS $_$
import sys
sys.path.append('/home/malware')
import httputils
import re
import urllib
import urlparse
from pg import unescape_bytea

def process_match(str, provider):
    if not SD.has_key('link_r'):
        SD['link_r'] = re.compile('.*<a href.*id=([0-9]+)\&amp\;password=([a-z]+).*', re.M)
    link_m = SD['link_r'].search(str)
    if link_m != None:
        id = link_m.group(1)
        passwd = link_m.group(2)
        if not SD.has_key('insert'):
            q = "INSERT INTO reports (md5, provider, analysis_id, password, available) VALUES ($1, $2, $3, $4, true)"
            SD['insert'] = plpy.prepare(q, [ "text", "provider", "int4" , "text"])
        plpy.execute(SD['insert'], [ md5, provider, id, passwd ])
        return True         
    else:
        plpy.warning('failed link match')
        return False

if not SD.has_key('selcheck'):
    SD['selcheck'] = plpy.prepare(
        "SELECT md5, provider FROM reports WHERE md5 = $1 and (provider = 'CWSandbox' or provider='VirusTotal')",
        [ "text" ])

res = plpy.execute(SD['selcheck'], [ md5 ])
if len(res) == 1:
    skip_provider = res[0]['provider']
elif len(res) > 1:
    return 'false'
else:
    skip_provider = None

if not SD.has_key('submit_url'):
    res = plpy.execute("select value as submit_url from settings where key = 'CWSANDBOX_SUBMIT_URL'")
    SD['submit_url'] = res[0]['submit_url']

if not SD.has_key('sample_check'):
    SD['sample_check'] = plpy.prepare('SELECT sample FROM samples WHERE md5 = $1 AND sample IS NOT NULL', [ "text" ])

res = plpy.execute(SD['sample_check'], [ md5 ])
if len(res) > 0:
    file = md5
    content = unescape_bytea(res[0]['sample'])
else:
    file = "/var/www/uploads/%s" %(md5)
    fd = open(file, "rb")
    content = fd.read()
    fd.close()

post_data = {}
post_data['email'] = 'malware@example.com'
post_data["upfile"] = { "content" : content, "filename" : file }
response = httputils.httprequest(SD['submit_url'], post_data)
r = response.read()
if r.startswith('http://'):
    url = r
else:
    banner_r = re.compile('.*Please take a look at.*id=([0-9]+)&amp;password=([a-z]+).*',re.M|re.S)
    m = banner_r.search(r)
    if m != None:
        url = 'http://www.cwsandbox.org/?page=samdet&id=' + m.group(1) + '&password=' + m.group(2)
    elif r.find("Submission added. Thank you. Upon completion of the analysis you will find it on") != -1:
        plpy.notice("new sample sumbitted for analysis")
        return 'true'
    elif r.find("You have already submitted this sample. Upon completion of the analysis you will find it on") != -1:
        plpy.notice('old sample, analysis in progress')
        return 'true'
    else:
        plpy.notice('unknown reply')
        plpy.notice(r)
        return 'true'
plpy.notice(url)
f = urllib.urlopen(url)
s = f.read()
f.close()
val = 'false'
idx = s.find('>Analysis</a>')
if idx != -1:
    virustotal_r = re.compile('VirusTotal Scan (.*)Analysis</a>',re.S)
    cwsandbox_r = re.compile('CWSandbox [0-9]+(.*)Analysis</a>',re.S)
    plpy.notice("found page")
    cws_m = cwsandbox_r.search(s)
    if cws_m != None and skip_provider != 'CWSandbox':
        plpy.notice("cwsandbox match")
        str = cws_m.group(1)
        if str.find('Error #1: The sample is no valid Win32 application') != -1:
            plpy.notice("invalid sample")
            query = plpy.prepare("UPDATE samples SET type='invalid' WHERE md5 = $1",['text'])
            plpy.execute(query, [ md5 ])
        elif str.find('Error #255: An unknown error happened during analysis') != -1 or str.find('Error #3: An error happened during analysis') != -1:
            plpy.notice("analysis error")
        elif str.find('analysis running') != -1:
            plpy.notice("analysis running") 
        elif process_match(str, 'CWSandbox'):
            val = 'true'
    virus_m = virustotal_r.search(s)
    if virus_m != None and skip_provider != 'VirusTotal':
        plpy.notice("virustotal match")
        if process_match(virus_m.group(1), 'VirusTotal'):
            val = 'true'       
return val
$_$
    LANGUAGE plpythonu;

--
-- Name: update_cncs_geo(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION update_cncs_geo() RETURNS integer
    AS $_$
import GeoIP
if not SD.has_key('query'):
    SD['query'] = plpy.prepare('update cncs set country = $2, latitude = $3, longitude = $4 where id = $1', [ "int4", "text", "float8", "float8" ])
res = plpy.execute('select id, address from cncs')
g = GeoIP.open('/home/malware/GeoLiteCity.dat', GeoIP.GEOIP_STANDARD)
i=0
for row in res:
    data = g.record_by_addr(row['address'])
    if data != None:
        code = data['country_code']
        lat = data['latitude']
        lon = data['longitude']
        plpy.execute(SD['query'], [ row['id'], code, lat, lon ])
        i=i+1
return i

$_$
    LANGUAGE plpythonu;

--
-- Name: update_reports_availability(); Type: FUNCTION; Schema: public; Owner: malware
--

CREATE FUNCTION update_reports_availability() RETURNS integer
    AS $$
-- yes, it's working, it's just really slow...
DECLARE
    myrow RECORD;
    val boolean DEFAULT 'false';
    counter integer DEFAULT 0;
BEGIN
    FOR myrow IN SELECT id FROM reports WHERE available = 'false' LOOP
        RAISE NOTICE 'Checking report %', myrow.id;
        SELECT check_report_availability(myrow.id) INTO val;
--        RAISE NOTICE 'Result is %', val;
        IF val THEN
--            RAISE NOTICE 'Updating record %', myrow.id;
            UPDATE reports SET available = 'true' WHERE id = myrow.id;
            counter := counter + 1;
--            RAISE NOTICE 'Counter is now %', counter;
        END IF;
    END LOOP;
    RETURN counter;
END;
$$
    LANGUAGE plpgsql;

--
-- Name: av_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE av_reports_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: av_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE av_reports_id_seq OWNED BY av_reports.id;


--
-- Name: http_cmds_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE http_cmds_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: http_cmds_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE http_cmds_id_seq OWNED BY http_cmds.id;


--
-- Name: http_replies_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE http_replies_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: http_replies_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE http_replies_id_seq OWNED BY http_replies.id;


--
-- Name: irc_channels_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE irc_channels_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: irc_channels_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE irc_channels_id_seq OWNED BY irc_channels.id;


--
-- Name: irc_commands_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE irc_commands_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: irc_commands_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE irc_commands_id_seq OWNED BY irc_commands.id;


--
-- Name: irc_notices_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE irc_notices_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: irc_notices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE irc_notices_id_seq OWNED BY irc_privmsgs.id;


--
-- Name: reports_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE reports_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
    
--
-- Name: reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE reports_id_seq OWNED BY reports.id;


--
-- Name: sightings_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE sightings_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: sightings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE sightings_id_seq OWNED BY sightings.id;


--
-- Name: targets_id_seq; Type: SEQUENCE; Schema: public; Owner: malware
--

CREATE SEQUENCE targets_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: targets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: malware
--

ALTER SEQUENCE targets_id_seq OWNED BY targets.id;


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE av_reports ALTER COLUMN id SET DEFAULT nextval('av_reports_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE http_cmds ALTER COLUMN id SET DEFAULT nextval('http_cmds_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE http_replies ALTER COLUMN id SET DEFAULT nextval('http_replies_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE irc_channels ALTER COLUMN id SET DEFAULT nextval('irc_channels_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE irc_commands ALTER COLUMN id SET DEFAULT nextval('irc_commands_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE irc_privmsgs ALTER COLUMN id SET DEFAULT nextval('irc_notices_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE reports ALTER COLUMN id SET DEFAULT nextval('reports_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE sightings ALTER COLUMN id SET DEFAULT nextval('sightings_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: malware
--

ALTER TABLE targets ALTER COLUMN id SET DEFAULT nextval('targets_id_seq'::regclass);


--
-- Name: antivirus_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY av_reports
    ADD CONSTRAINT antivirus_reports_pkey PRIMARY KEY (id);


--
-- Name: antivirus_reports_uc2; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY av_reports
    ADD CONSTRAINT antivirus_reports_uc2 UNIQUE (md5, scanner);


--
-- Name: clamav_variants_pk; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY clamav_variants
    ADD CONSTRAINT clamav_variants_pk PRIMARY KEY (name, variant);


--
-- Name: cnc_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY cncs
    ADD CONSTRAINT cnc_pkey PRIMARY KEY (id);


--
-- Name: cnc_uc2; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY cncs
    ADD CONSTRAINT cnc_uc2 UNIQUE (address, port);


--
-- Name: http_bots_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY http_bots
    ADD CONSTRAINT http_bots_pkey PRIMARY KEY (id);


--
-- Name: http_bots_ref_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY http_bots_ref
    ADD CONSTRAINT http_bots_ref_pkey PRIMARY KEY (http_bot, http_cmd);


--
-- Name: http_bots_uc3; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY http_bots
    ADD CONSTRAINT http_bots_uc3 UNIQUE (cnc);


--
-- Name: http_cmds_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY http_cmds
    ADD CONSTRAINT http_cmds_pkey PRIMARY KEY (id);


--
-- Name: http_replies_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY http_replies
    ADD CONSTRAINT http_replies_pkey PRIMARY KEY (id);


--
-- Name: irc_bot_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_bots
    ADD CONSTRAINT irc_bot_pkey PRIMARY KEY (id);


--
-- Name: irc_bots_ref_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_bots_refc
    ADD CONSTRAINT irc_bots_ref_pkey PRIMARY KEY (irc_bot, irc_channel);


--
-- Name: irc_bots_refp_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_bots_refp
    ADD CONSTRAINT irc_bots_refp_pkey PRIMARY KEY (irc_bot, irc_privmsg);


--
-- Name: irc_channels_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_channels
    ADD CONSTRAINT irc_channels_pkey PRIMARY KEY (id);


--
-- Name: irc_commands_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_commands
    ADD CONSTRAINT irc_commands_pkey PRIMARY KEY (id);


--
-- Name: irc_notices_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY irc_privmsgs
    ADD CONSTRAINT irc_notices_pkey PRIMARY KEY (id);


--
-- Name: malware_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY samples
    ADD CONSTRAINT malware_pkey PRIMARY KEY (md5);


--
-- Name: reports_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_pkey PRIMARY KEY (id);


--
-- Name: reports_uc5; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_uc5 UNIQUE (md5, provider);


--
-- Name: reports_uc6; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_uc6 UNIQUE (task_id);


--
-- Name: reports_uc7; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_uc7 UNIQUE (analysis_id);


--
-- Name: samples_ref_pk; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY samples_ref
    ADD CONSTRAINT samples_ref_pk PRIMARY KEY (sample, malware_id);


--
-- Name: sensors_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY sensors
    ADD CONSTRAINT sensors_pkey PRIMARY KEY (name);


--
-- Name: settings_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY settings
    ADD CONSTRAINT settings_pkey PRIMARY KEY (key);


--
-- Name: sightings_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY sightings
    ADD CONSTRAINT sightings_pkey PRIMARY KEY (id);


--
-- Name: targets_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY targets
    ADD CONSTRAINT targets_pkey PRIMARY KEY (id);


--
-- Name: worms_pkey; Type: CONSTRAINT; Schema: public; Owner: malware; Tablespace: 
--

ALTER TABLE ONLY worms
    ADD CONSTRAINT worms_pkey PRIMARY KEY (id);


--
-- Name: av_reports_idx3; Type: INDEX; Schema: public; Owner: malware; Tablespace: 
--

CREATE INDEX av_reports_idx3 ON av_reports USING btree (md5);


--
-- Name: INDEX av_reports_idx3; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON INDEX av_reports_idx3 IS 'Antivirus reports are usually processed by sample MD5.';


--
-- Name: reports_idx_md5; Type: INDEX; Schema: public; Owner: malware; Tablespace: 
--

CREATE INDEX reports_idx_md5 ON reports USING btree (md5);

--
-- Name: do_submit; Type: TRIGGER; Schema: public; Owner: malware
--

CREATE TRIGGER do_submit
    AFTER INSERT ON samples
    FOR EACH ROW
    EXECUTE PROCEDURE do_submit();


--
-- Name: set_geo; Type: TRIGGER; Schema: public; Owner: malware
--

CREATE TRIGGER set_geo
    BEFORE INSERT ON cncs
    FOR EACH ROW
    EXECUTE PROCEDURE set_geo();


--
-- Name: TRIGGER set_geo ON cncs; Type: COMMENT; Schema: public; Owner: malware
--

COMMENT ON TRIGGER set_geo ON cncs IS 'Sets the fields country, latitude, longitude according to the IP address location.';


--
-- Name: http_bots_cnc; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_bots
    ADD CONSTRAINT http_bots_cnc FOREIGN KEY (cnc) REFERENCES cncs(id);


--
-- Name: http_bots_ref_bot; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_bots_ref
    ADD CONSTRAINT http_bots_ref_bot FOREIGN KEY (http_bot) REFERENCES http_bots(id);


--
-- Name: http_bots_ref_cmd; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_bots_ref
    ADD CONSTRAINT http_bots_ref_cmd FOREIGN KEY (http_cmd) REFERENCES http_cmds(id);


--
-- Name: http_replies_http_bot; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_replies
    ADD CONSTRAINT http_replies_http_bot FOREIGN KEY (http_bot) REFERENCES http_bots(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: http_replies_http_cmd; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_replies
    ADD CONSTRAINT http_replies_http_cmd FOREIGN KEY (http_cmd) REFERENCES http_cmds(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: http_replies_sensor; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY http_replies
    ADD CONSTRAINT http_replies_sensor FOREIGN KEY (sensor) REFERENCES sensors(name);


--
-- Name: irc_bot_cnc; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_bots
    ADD CONSTRAINT irc_bot_cnc FOREIGN KEY (cnc) REFERENCES cncs(id);


--
-- Name: irc_bots_ref_bot; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_bots_refc
    ADD CONSTRAINT irc_bots_ref_bot FOREIGN KEY (irc_bot) REFERENCES irc_bots(id);


--
-- Name: irc_bots_ref_channel; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_bots_refc
    ADD CONSTRAINT irc_bots_ref_channel FOREIGN KEY (irc_channel) REFERENCES irc_channels(id);


--
-- Name: irc_bots_refp_bot; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_bots_refp
    ADD CONSTRAINT irc_bots_refp_bot FOREIGN KEY (irc_bot) REFERENCES irc_bots(id);


--
-- Name: irc_bots_refp_privmsg; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_bots_refp
    ADD CONSTRAINT irc_bots_refp_privmsg FOREIGN KEY (irc_privmsg) REFERENCES irc_privmsgs(id);


--
-- Name: irc_commands_bot; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_commands
    ADD CONSTRAINT irc_commands_bot FOREIGN KEY (irc_bot) REFERENCES irc_bots(id);


--
-- Name: irc_commands_channel; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_commands
    ADD CONSTRAINT irc_commands_channel FOREIGN KEY (irc_channel) REFERENCES irc_channels(id);


--
-- Name: irc_commands_sensor; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY irc_commands
    ADD CONSTRAINT irc_commands_sensor FOREIGN KEY (sensor) REFERENCES sensors(name);


--
-- Name: reports_md5; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_md5 FOREIGN KEY (md5) REFERENCES samples(md5);


--
-- Name: sightings_md5; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY sightings
    ADD CONSTRAINT sightings_md5 FOREIGN KEY (md5) REFERENCES samples(md5) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sightings_sensor; Type: FK CONSTRAINT; Schema: public; Owner: malware
--

ALTER TABLE ONLY sightings
    ADD CONSTRAINT sightings_sensor FOREIGN KEY (sensor) REFERENCES sensors(name);


