#!/usr/bin/env python
#adapted from http://anubis.iseclab.org/Resources/submit_to_anubis.py
UPLOAD_URL = "http://gateway.hive.example.com/upload.php"

import urllib
import sys
import httplib
import optparse
import hashlib

usage_msg="%prog [options] sample sample2 sample3...\nStart with --help for more information."
parser = optparse.OptionParser(usage=usage_msg)
parser.add_option("-e", "--experiment", dest="experiment", type="string", help="")
parser.add_option("-s", "--sensor", dest="sensor", type="string", help="")
parser.add_option("-n", "--name", dest="name", type="string", help="")
parser.add_option("-p", "--path", dest="path", type="string", help="")
parser.add_option("-d", "--date", dest="date", type="string", help="")
parser.add_option("-z", "--size", dest="size", type="string", help="")
empty_options = parser.parse_args()[0]

#This function was copied from an email of Trent Mick and
#afterwards modified
def httprequest(url, postdata={}, headers={}):
    """A urllib.urlopen() replacement for http://... that gets the
    content-type right for multipart POST requests.

    "url" is the http URL to open.
    "postdata" is a dictionary describing data to post. If the dict is
        empty (the default) a GET request is made, otherwise a POST
        request is made. Each postdata item maps a string name to
        either:
        - a string value; or
        - a file part specification of the form:
            {"filename": <filename>,    # file to load content from
             "content": <content>,      # (optional) file content
             "headers": <headers>}      # (optional) headers
          <filename> is used to load the content (can be overridden by
          <content>) and as the filename to report in the request.
          <headers> is a dictionary of headers to use for the part.
          Note: currently the file part content but be US-ASCII text.
    "headers" is an optional dictionary of headers to send with the
        request. Note that the "Content-Type" and "Content-Length"
        headers are automatically determined.

    The current urllib.urlopen() *always* uses:
        Content-Type: application/x-www-form-urlencoded
    for POST requests. This is incorrect if the postdata includes a file
    to upload. If a file is to be posted the post data is:
        Content-Type: multipart/form-data
    
    This returns the response content if the request was successfull
    (HTTP code 200). Otherwise an IOError is raised.

    For example, this invocation:
        url = 'http://www.perl.org/survey.cgi'
        postdata = {
            "name": "Gisle Aas",
            "email": "gisle at aas.no",
            "gender": "M",
            "born": "1964",
            "init": {"filename": "~/.profile"},
        }
   
    Inspiration: Perl's HTTP::Request module.
    http://aspn.activestate.com/ASPN/Reference/Products/ActivePerl/site/lib/HTTP/Request/Common.html
    """
    import httplib, urllib, urlparse
    from email.MIMEText import MIMEText
    from email.MIMEMultipart import MIMEMultipart
    from email import Encoders
    from email.MIMEBase import MIMEBase
    
    if not url.startswith("http://"):
        raise "Invalid URL, only http:// URLs are allow: url='%s'" % url

    if not postdata:
        method = "GET"
        body = None
    else:
        method = "POST"

        # Determine if require a multipart content-type: 'contentType'.
        for part in postdata.values():
            if isinstance(part, dict):
                contentType = "multipart/form-data"
                break
        else:
            contentType = "application/x-www-form-urlencoded"
        headers["Content-Type"] = contentType

        # Encode the post data: 'body'.
        if contentType == "application/x-www-form-urlencoded":
            body = urllib.urlencode(postdata)
        elif contentType == "multipart/form-data":
            message = MIMEMultipart(_subtype="form-data")
            for name, value in postdata.items():
                if isinstance(value, dict):
                    # Get content.
                    if "content" in value:
                        content = value["content"]
                    else:
                        fp = open(value["filename"], "rb")
                        content = fp.read()

                    part = MIMEBase('application', "octet-stream")
                    part.set_payload( content )
#                    Encoders.encode_base64(part)

                    # Add content-disposition header.
                    dispHeaders = value.get("headers", {})
                    if "Content-Disposition" not in dispHeaders:
                        #XXX Should be a case-INsensitive check.
                        part.add_header("Content-Disposition", "form-data",
                                        name=name, filename=value["filename"])
                    for dhName, dhValue in dispHeaders:
                        part.add_header(dhName, dhValue)
                else:
                    # Do not use ctor to set payload to avoid adding a
                    # trailing newline.
                    part = MIMEText(None)
                    part.set_payload(value, "us-ascii")
                    part.add_header("Content-Disposition", "form-data",
                                    name=name)
                message.attach(part)
            message.epilogue = "" # Make sure body ends with a newline.
            # Split off the headers block from the .as_string() to get
            # just the message content. Also add the multipart Message's
            # headers (mainly to get the Content-Type header _with_ the
            # boundary attribute).
            headerBlock, body = message.as_string().split("\n\n",1)
            for hName, hValue in message.items():
                headers[hName] = hValue
        else:
            raise "Invalid content-type: '%s'" % contentType

    # Make the HTTP request and get the response.
    # Precondition: 'url', 'method', 'headers', 'body' are all setup properly.
    scheme, netloc, path, parameters, query, fragment = urlparse.urlparse(url)
    if parameters or query or fragment:
        raise "Unexpected URL form: parameters, query or fragment parts "\
              "are not allowed: parameters=%r, query=%r, fragment=%r"\
              % (parameters, query, fragment)
    conn = httplib.HTTPConnection(netloc)
    conn.request(method, path, body, headers)
    response = conn.getresponse()
    return response
    
def submit(file_arg, experiment, sensor, filename, filepath, filedate, filesize):
    try:
        if (file_arg == "-"):
            content = sys.stdin.read()
            fn = "stdin"
        else:
            content = open(file_arg, "rb").read()
            fn = file_arg
        
        post_data = {}
	post_data["MAX_FILE_SIZE"] = "10000000"
	hasher = hashlib.md5()
	hasher.update(content)
	post_data["md5"] = hasher.hexdigest()
        post_data["experiment"] = experiment.replace('-', ' ')
        post_data["sensor"] = sensor
        post_data["filename"] = filename
	post_data["filepath"] = filepath
	post_data["filedate"] = filedate
	post_data["filesize"] = filesize
        post_data["userfile"] = {"content" : content, "filename" : fn}

        response = httprequest(UPLOAD_URL, post_data)
	print response.read()
    except Exception:
        import traceback
        traceback.print_exc()
    return 

def main(file_arg, options = empty_options):
    submit(file_arg, options.experiment, options.sensor, options.name, options.path, options.date, options.size)

if __name__ == '__main__':
    (options, args) = parser.parse_args()

    if (len(args) == 0):
        parser.print_usage()
        sys.exit(2)

    for filename in args:
        main(filename, options)

